from ._utils import api_util
from .Bases import Iterable
from dss_python_backend.enums import StorageStates

class IStorages(Iterable):
    """Storage objects"""

    __name__ = "Storages"
    _api_prefix = "Storages"
    __slots__ = []
    _columns = [
        "Name",
        "Idx",
        "RegisterNames",
        "RegisterValues",
        "puSOC",
        "State",
        "AmpLimit",
        "AmpLimitGain",
        "ChargeTrigger",
        "ControlMode",
        "DischargeTrigger",
        "EffCharge",
        "EffDischarge",
        "Kp",
        "kV",
        "kVA",
        "kvar",
        "kVDC",
        "kW",
        "kWhRated",
        "kWRated",
        "LimitCurrent",
        "PF",
        "PITol",
        "SafeMode",
        "SafeVoltage",
        "TimeChargeTrig",
        "VarFollowInverter",
    ]

    def puSOC(self, *args):
        """Per unit state of charge"""
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_puSOC()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_puSOC(Value)

    def State(self, *args):
        """
        Get/set state: 0=Idling; 1=Discharging; -1=Charging;
        """
        # Getter
        if len(args) == 0:
            return StorageStates(self._lib.Storages_Get_State())

        # Setter
        (Value,) = args
        self._lib.Storages_Set_State(Value)

    def RegisterNames(self):
        """
        Array of Storage energy meter register names

        See also the enum `GeneratorRegisters`.
        """
        return self._lib.Storages_Get_RegisterNames()

    def RegisterValues(self):
        """Array of values in Storage registers."""
        return self._lib.Storages_Get_RegisterValues_GR()

    def AmpLimit(self, *args):
        """
        Current limit per phase for the IBR when operating in GFM mode.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_AmpLimit()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_AmpLimit(Value)

    def AmpLimitGain(self, *args):
        """
        Use it for fine tuning the current limiter when active.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_AmpLimitGain()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_AmpLimitGain(Value)

    def ChargeTrigger(self, *args):
        """
        Dispatch trigger value for charging the Storage.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_ChargeTrigger()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_ChargeTrigger(Value)

    def ControlMode(self, *args):
        """
        Control mode for the inverter. It can be one of {GFM = 1 | GFL* = 0}.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_ControlMode()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_ControlMode(Value)

    def DischargeTrigger(self, *args):
        """
        Dispatch trigger value for discharging the Storage.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_DischargeTrigger()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_DischargeTrigger(Value)

    def EffCharge(self, *args):
        """
        Percentage efficiency for CHARGING the Storage element.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_EffCharge()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_EffCharge(Value)

    def EffDischarge(self, *args):
        """
        Percentage efficiency for DISCHARGING the Storage element.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_EffDischarge()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_EffDischarge(Value)

    def Kp(self, *args):
        """
        Proportional gain for the PI controller within the inverter.
        Use it to modify the controller response in dynamics simulation mode.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_Kp()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_Kp(Value)

    def kV(self, *args):
        """
        Nominal rated (1.0 per unit) voltage, kV, for Storage element.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kV()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kV(Value)

    def kVA(self, *args):
        """
        Inverter nameplate capability (in kVA). Used as the base for Dynamics mode and Harmonics mode values.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kVA()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kVA(Value)

    def kvar(self, *args):
        """
        Get/set the requested kvar value. Final kvar is subjected to the inverter ratings. Sets inverter to operate in constant kvar mode.

        **Note:** reading the DSS property `kvar` returns the adjusted value, while this returns the original input value.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kvar()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kvar(Value)

    def kVDC(self, *args):
        """
        Rated voltage (kV) at the input of the inverter while the storage is discharging
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kVDC()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kVDC(Value)

    def kW(self, *args):
        """
        Get/set the requested kW value. Final kW is subjected to the inverter ratings.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kW()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kW(Value)

    def kWhRated(self, *args):
        """
        Rated Storage capacity in kWh.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kWhRated()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kWhRated(Value)

    def kWRated(self, *args):
        """
        kW rating of power output. Base for Loadshapes when DispMode=Follow. Sets kVA property if it has not been specified yet.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_kWRated()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_kWRated(Value)

    def LimitCurrent(self, *args):
        """
        Limits current magnitude to Vminpu value for both 1-phase and 3-phase Storage similar to Generator Model 7.
        For 3-phase, limits the positive-sequence current but not the negative-sequence."
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_LimitCurrent()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_LimitCurrent(Value)

    def PF(self, *args):
        """
        Get/set the requested PF value.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_PF()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_PF(Value)

    def PITol(self, *args):
        """
        Tolerance (%) for the closed loop controller of the inverter
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_PITol()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_PITol(Value)

    def SafeMode(self):
        """
        (Read only) Indicates whether the inverter entered (Yes) or not (No) into Safe Mode.
        """
        return self._lib.Storages_Get_SafeMode()

    def SafeVoltage(self, *args):
        """
        Indicates the voltage level (%) respect to the base voltage level for which the Inverter will operate.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_SafeVoltage()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_SafeVoltage(Value)

    def TimeChargeTrig(self, *args):
        """
        Time of day in fractional hours (0230 = 2.5) at which Storage element will automatically go into charge state.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_TimeChargeTrig()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_TimeChargeTrig(Value)

    def VarFollowInverter(self, *args):
        """
        Indicates if the reactive power generation/absorption does not respect the inverter status
        """
        # Getter
        if len(args) == 0:
            return self._lib.Storages_Get_VarFollowInverter()

        # Setter
        (Value,) = args
        self._lib.Storages_Set_VarFollowInverter(Value)


_Storages = IStorages(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Storages.AllNames
AmpLimit = _Storages.AmpLimit
AmpLimitGain = _Storages.AmpLimitGain
ChargeTrigger = _Storages.ChargeTrigger
ControlMode = _Storages.ControlMode
Count = _Storages.Count
DischargeTrigger = _Storages.DischargeTrigger
EffCharge = _Storages.EffCharge
EffDischarge = _Storages.EffDischarge
First = _Storages.First
Idx = _Storages.Idx
Kp = _Storages.Kp
LimitCurrent = _Storages.LimitCurrent
Name = _Storages.Name
Next = _Storages.Next
PF = _Storages.PF
PITol = _Storages.PITol
RegisterNames = _Storages.RegisterNames
RegisterValues = _Storages.RegisterValues
SafeMode = _Storages.SafeMode
SafeVoltage = _Storages.SafeVoltage
State = _Storages.State
TimeChargeTrig = _Storages.TimeChargeTrig
VarFollowInverter = _Storages.VarFollowInverter
kV = _Storages.kV
kVA = _Storages.kVA
kVDC = _Storages.kVDC
kW = _Storages.kW
kWRated = _Storages.kWRated
kWhRated = _Storages.kWhRated
kvar = _Storages.kvar
puSOC = _Storages.puSOC
_columns = _Storages._columns
__all__ = [
    "AllNames",
    "AmpLimit",
    "AmpLimitGain",
    "ChargeTrigger",
    "ControlMode",
    "Count",
    "DischargeTrigger",
    "EffCharge",
    "EffDischarge",
    "First",
    "Idx",
    "Kp",
    "LimitCurrent",
    "Name",
    "Next",
    "PF",
    "PITol",
    "RegisterNames",
    "RegisterValues",
    "SafeMode",
    "SafeVoltage",
    "State",
    "TimeChargeTrig",
    "VarFollowInverter",
    "kV",
    "kVA",
    "kVDC",
    "kW",
    "kWRated",
    "kWhRated",
    "kvar",
    "puSOC",
]



from ._utils import api_util
from .Bases import Base


class IBus(Base):
    __slots__ = []

    __name__ = "Bus"
    _api_prefix = "Bus"
    _columns = [
        "Name",
        "NumNodes",
        "Nodes",
        "Coorddefined",
        "CplxSeqVoltages",
        "Cust_Duration",
        "Cust_Interrupts",
        "Cust_Interrupts",
        "Distance",
        "Int_Duration",
        "Isc",
        "Lambda",
        "N_Customers",
        "N_interrupts",
        "SectionID",
        "SeqVoltages",
        "TotalMiles",
        "VLL",
        "VMagAngle",
        "Voc",
        "Voltages",
        "YscMatrix",
        "Zsc0",
        "Zsc1",
        "ZscMatrix",
        "kVBase",
        "puVLL",
        "puVmagAngle",
        "PuVoltage",
        "X",
        "Y",
        "AllPCEatBus",
        "AllPDEatBus",
    ]

    def GetUniqueNodeNumber(self, StartNumber):
        """
        Return a unique node number at the active bus to avoid node collisions and adds
        it to the node list for the bus.

        Original COM help: https://opendss.epri.com/GetUniqueNodeNumber.html
        """
        return self._lib.Bus_GetUniqueNodeNumber(StartNumber)

    def ZscRefresh(self):
        """
        Refreshes the Zsc matrix for the active bus.

        Original COM help: https://opendss.epri.com/ZscRefresh.html
        """
        return self._lib.Bus_ZscRefresh()

    def Coorddefined(self):
        """
        Indicates whether a coordinate has been defined for this bus

        Original COM help: https://opendss.epri.com/Coorddefined.html
        """
        return self._lib.Bus_Get_Coorddefined()

    def CplxSeqVoltages(self):
        """
        Complex array of Sequence Voltages (0, 1, 2) at this Bus.

        Original COM help: https://opendss.epri.com/CplxSeqVoltages.html
        """
        return self._lib.Bus_Get_CplxSeqVoltages_GR()

    def Cust_Duration(self):
        """
        Accumulated customer outage durations

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/Cust_Duration.html
        """
        return self._lib.Bus_Get_Cust_Duration()

    def Cust_Interrupts(self):
        """
        Annual number of customer-interruptions from this bus

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/Cust_Interrupts.html
        """
        return self._lib.Bus_Get_Cust_Interrupts()

    def Distance(self):
        """
        Distance from EnergyMeter (if non-zero)

        *Requires an energy meter with an updated zone.*

        Original COM help: https://opendss.epri.com/Distance.html
        """
        return self._lib.Bus_Get_Distance()

    def Int_Duration(self):
        """
        Average interruption duration, hours.

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/Int_Duration.html
        """
        return self._lib.Bus_Get_Int_Duration()

    def Isc(self):
        """
        Short circuit currents at bus; Complex Array.

        *Requires a previous solution in `FaultStudy` mode.*

        Original COM help: https://opendss.epri.com/Isc.html
        """
        return self._lib.Bus_Get_Isc_GR()

    def Lambda(self):
        """
        Accumulated failure rate downstream from this bus; faults per year

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/Lambda.html
        """
        return self._lib.Bus_Get_Lambda()

    def N_Customers(self):
        """
        Total numbers of customers served downline from this bus

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/N_Customers.html
        """
        return self._lib.Bus_Get_N_Customers()

    def N_interrupts(self):
        """
        Number of interruptions this bus per year

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/N_interrupts.html
        """
        return self._lib.Bus_Get_N_interrupts()

    def Name(self):
        """
        Name of the active Bus

        Original COM help: https://opendss.epri.com/Name1.html
        """
        return self._lib.Bus_Get_Name()

    def Nodes(self):
        """
        Integer Array of Node Numbers defined at the bus in same order as the voltages.

        Original COM help: https://opendss.epri.com/Nodes.html
        """
        return self._lib.Bus_Get_Nodes_GR()

    def NumNodes(self):
        """
        Number of Nodes this bus.

        Original COM help: https://opendss.epri.com/NumNodes.html
        """
        return self._lib.Bus_Get_NumNodes()

    def SectionID(self):
        """
        Integer ID of the feeder section in which this bus is located.

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/SectionID.html
        """
        return self._lib.Bus_Get_SectionID()

    def SeqVoltages(self):
        """
        Double Array of sequence voltages at this bus. Magnitudes only.

        Original COM help: https://opendss.epri.com/SeqVoltages.html
        """
        return self._lib.Bus_Get_SeqVoltages_GR()

    def TotalMiles(self):
        """
        Total length of line downline from this bus, in miles. For recloser siting algorithm.

        *Requires a previous call to `RelCalc` command*

        Original COM help: https://opendss.epri.com/TotalMiles.html
        """
        return self._lib.Bus_Get_TotalMiles()

    def VLL(self):
        """
        For 2- and 3-phase buses, returns array of complex numbers representing L-L voltages in volts. Returns -1.0 for 1-phase bus. If more than 3 phases, returns only first 3.

        Original COM help: https://opendss.epri.com/VLL.html
        """
        return self._lib.Bus_Get_VLL_GR()

    def VMagAngle(self):
        """
        Array of doubles containing voltages in Magnitude (VLN), angle (degrees)

        Original COM help: https://opendss.epri.com/VMagAngle.html
        """
        return self._lib.Bus_Get_VMagAngle_GR()

    def Voc(self):
        """
        Open circuit voltage; Complex array.

        *Requires a previous solution in `FaultStudy` mode.*

        Original COM help: https://opendss.epri.com/Voc.html
        """
        return self._lib.Bus_Get_Voc_GR()

    def Voltages(self):
        """
        Complex array of voltages at this bus.

        Original COM help: https://opendss.epri.com/Voltages.html
        """
        return self._lib.Bus_Get_Voltages_GR()

    def YscMatrix(self):
        """
        Complex array of Ysc matrix at bus. Column by column.

        *Requires a previous solution in `FaultStudy` mode or a call to `ZscRefresh`.*

        Original COM help: https://opendss.epri.com/YscMatrix.html
        """
        return self._lib.Bus_Get_YscMatrix_GR()

    def Zsc0(self):
        """
        Complex Zero-Sequence short circuit impedance at bus.

        *Requires a previous solution in `FaultStudy` mode or a call to `ZscRefresh`.*

        Original COM help: https://opendss.epri.com/Zsc0.html
        """
        return self._lib.Bus_Get_Zsc0_GR()

    def Zsc1(self):
        """
        Complex Positive-Sequence short circuit impedance at bus.

        *Requires a previous solution in `FaultStudy` mode or a call to `ZscRefresh`.*

        Original COM help: https://opendss.epri.com/Zsc1.html
        """
        return self._lib.Bus_Get_Zsc1_GR()

    def ZscMatrix(self):
        """
        Complex array of Zsc matrix at bus. Column by column.

        *Requires a previous solution in `FaultStudy` mode or a call to `ZscRefresh`.*

        Original COM help: https://opendss.epri.com/ZscMatrix.html
        """
        return self._lib.Bus_Get_ZscMatrix_GR()

    def kVBase(self):
        """
        Base voltage at bus in kV

        Original COM help: https://opendss.epri.com/kVBase.html
        """
        return self._lib.Bus_Get_kVBase()

    def puVLL(self):
        """
        Returns Complex array of pu L-L voltages for 2- and 3-phase buses. Returns -1.0 for 1-phase bus. If more than 3 phases, returns only 3 phases.

        Original COM help: https://opendss.epri.com/puVLL.html
        """
        return self._lib.Bus_Get_puVLL_GR()

    def puVmagAngle(self):
        """
        Array of doubles containing voltage magnitude, angle (degrees) pairs in per unit

        Original COM help: https://opendss.epri.com/puVmagAngle.html
        """
        return self._lib.Bus_Get_puVmagAngle_GR()

    def PuVoltage(self):
        """
        Complex Array of pu voltages at the bus.

        Original COM help: https://opendss.epri.com/puVoltages.html
        """
        return self._lib.Bus_Get_puVoltages_GR()

    def ZSC012Matrix(self):
        """
        Complex array containing the complete 012 Zsc matrix.
        Only available after Zsc is computed, either through the "ZscRefresh" command, or running a "FaultStudy" solution.
        Only available for buses with 3 nodes.

        *Requires a previous solution in `FaultStudy` mode or a call to `ZscRefresh`.*

        Original COM help: https://opendss.epri.com/ZSC012Matrix.html
        """
        return self._lib.Bus_Get_ZSC012Matrix_GR()

    def X(self, *args):
        """
        X Coordinate for bus

        Original COM help: https://opendss.epri.com/x.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Bus_Get_x()

        # Setter
        (Value,) = args
        self._lib.Bus_Set_x(Value)

    def Y(self, *args):
        """
        Y coordinate for bus

        Original COM help: https://opendss.epri.com/y.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Bus_Get_y()

        # Setter
        (Value,) = args
        self._lib.Bus_Set_y(Value)

    def LoadList(self):
        """
        List of strings: Full Names of LOAD elements connected to the active bus.

        Original COM help: https://opendss.epri.com/LoadList.html
        """
        return self._lib.Bus_Get_LoadList()

    def LineList(self):
        """
        List of strings: Full Names of LINE elements connected to the active bus.

        Original COM help: https://opendss.epri.com/LineList.html
        """
        return self._lib.Bus_Get_LineList()

    def AllPCEatBus(self):
        """
        Returns an array with the names of all PCE connected to the active bus

        This also includes shunt Capacitors/Reactors.

        Original COM help: https://opendss.epri.com/AllPCEatBus.html
        """
        return self._lib.Bus_Get_AllPCEatBus()

    def AllPDEatBus(self):
        """
        Returns an array with the names of all PDE connected to the active bus

        This excludes shunt Capacitors/Reactors.

        Original COM help: https://opendss.epri.com/AllPDEatBus1.html
        """
        return self._lib.Bus_Get_AllPDEatBus()

    def to_altdss(self):
        """
        Returns a Python object for the current active bus in the circuit (if any).

        Requires AltDSS-Python.

        *Available only for the AltDSS engine.*

        **(API Extension)**

        """
        idx = self._lib.Bus_Get_idx()
        if idx < 0:
            return None
        return self._api_util.get_bus_obj(self._lib.Alt_Bus_GetByIndex(idx))


_Bus = IBus(api_util)

# For backwards compatibility, bind to the default instance
AllPCEatBus = _Bus.AllPCEatBus
AllPDEatBus = _Bus.AllPDEatBus
Coorddefined = _Bus.Coorddefined
CplxSeqVoltages = _Bus.CplxSeqVoltages
Cust_Duration = _Bus.Cust_Duration
Cust_Interrupts = _Bus.Cust_Interrupts
Distance = _Bus.Distance
GetUniqueNodeNumber = _Bus.GetUniqueNodeNumber
Int_Duration = _Bus.Int_Duration
Isc = _Bus.Isc
Lambda = _Bus.Lambda
LineList = _Bus.LineList
LoadList = _Bus.LoadList
N_Customers = _Bus.N_Customers
N_interrupts = _Bus.N_interrupts
Name = _Bus.Name
Nodes = _Bus.Nodes
NumNodes = _Bus.NumNodes
PuVoltage = _Bus.PuVoltage
SectionID = _Bus.SectionID
SeqVoltages = _Bus.SeqVoltages
TotalMiles = _Bus.TotalMiles
VLL = _Bus.VLL
VMagAngle = _Bus.VMagAngle
Voc = _Bus.Voc
Voltages = _Bus.Voltages
X = _Bus.X
Y = _Bus.Y
YscMatrix = _Bus.YscMatrix
ZSC012Matrix = _Bus.ZSC012Matrix
Zsc0 = _Bus.Zsc0
Zsc1 = _Bus.Zsc1
ZscMatrix = _Bus.ZscMatrix
ZscRefresh = _Bus.ZscRefresh
kVBase = _Bus.kVBase
puVLL = _Bus.puVLL
puVmagAngle = _Bus.puVmagAngle
to_altdss = _Bus.to_altdss
_columns = _Bus._columns
__all__ = [
    "AllPCEatBus",
    "AllPDEatBus",
    "Coorddefined",
    "CplxSeqVoltages",
    "Cust_Duration",
    "Cust_Interrupts",
    "Distance",
    "GetUniqueNodeNumber",
    "Int_Duration",
    "Isc",
    "Lambda",
    "LineList",
    "LoadList",
    "N_Customers",
    "N_interrupts",
    "Name",
    "Nodes",
    "NumNodes",
    "PuVoltage",
    "SectionID",
    "SeqVoltages",
    "TotalMiles",
    "VLL",
    "VMagAngle",
    "Voc",
    "Voltages",
    "X",
    "Y",
    "YscMatrix",
    "ZSC012Matrix",
    "Zsc0",
    "Zsc1",
    "ZscMatrix",
    "ZscRefresh",
    "kVBase",
    "puVLL",
    "puVmagAngle",
    "to_altdss",
]



from ._utils import api_util
from .Bases import Iterable


class IVsources(Iterable):
    __slots__ = []

    __name__ = "Vsources"
    _api_prefix = "Vsources"
    _columns = ["Name", "Idx", "Phases", "BasekV", "AngleDeg", "Frequency", "PU"]

    def AngleDeg(self, *args):
        """
        Phase angle of first phase in degrees

        Original COM help: https://opendss.epri.com/AngleDeg1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Vsources_Get_AngleDeg()

        # Setter
        (Value,) = args
        self._lib.Vsources_Set_AngleDeg(Value)

    def BasekV(self, *args):
        """
        Source voltage in kV

        Original COM help: https://opendss.epri.com/BasekV.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Vsources_Get_BasekV()

        # Setter
        (Value,) = args
        self._lib.Vsources_Set_BasekV(Value)

    def Frequency(self, *args):
        """
        Source frequency in Hz

        Original COM help: https://opendss.epri.com/Frequency2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Vsources_Get_Frequency()

        # Setter
        (Value,) = args
        self._lib.Vsources_Set_Frequency(Value)

    def Phases(self, *args):
        """
        Number of phases

        Original COM help: https://opendss.epri.com/Phases3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Vsources_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.Vsources_Set_Phases(Value)

    def PU(self, *args):
        """
        Per-unit value of source voltage

        Original COM help: https://opendss.epri.com/pu.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Vsources_Get_pu()

        # Setter
        (Value,) = args
        self._lib.Vsources_Set_pu(Value)


_Vsources = IVsources(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Vsources.AllNames
AngleDeg = _Vsources.AngleDeg
BasekV = _Vsources.BasekV
Count = _Vsources.Count
First = _Vsources.First
Frequency = _Vsources.Frequency
Idx = _Vsources.Idx
Name = _Vsources.Name
Next = _Vsources.Next
PU = _Vsources.PU
Phases = _Vsources.Phases
_columns = _Vsources._columns
__all__ = [
    "AllNames",
    "AngleDeg",
    "BasekV",
    "Count",
    "First",
    "Frequency",
    "Idx",
    "Name",
    "Next",
    "PU",
    "Phases",
]



import numpy as np
from ._utils import api_util
from .Bases import Base
from dss import SparseSolverOptions


class IYMatrix(Base):
    """
    YMatrix provides access to some lower-level solution aspects.

    Part of this class is ported from the original OpenDSSDirect.DLL back in 2017, but 
    part is new. Since this is not exposed in the official COM API, it is marked as an extension.

    (**API Extension**)
    """

    __slots__ = []

    __name__ = "YMatrix"
    _api_prefix = "YMatrix"
    _columns = []

    def getYsparse(self, factor=True):
        """
        Return as (data, indices, indptr) that can fed into `scipy.sparse.csc_matrix`
        """

        api_util = self._api_util
        ffi = api_util.ffi
        lib = api_util.lib_unpatched

        nBus = ffi.new("uint32_t*")
        nBus[0] = 0
        nNz = ffi.new("uint32_t*")
        nNz[0] = 0
        ColPtr = ffi.new("int32_t**")
        RowIdxPtr = ffi.new("int32_t**")
        cValsPtr = ffi.new("double**")
        lib.YMatrix_GetCompressedYMatrix(
            api_util.ctx, factor, nBus, nNz, ColPtr, RowIdxPtr, cValsPtr
        )
        if not nBus[0] or not nNz[0]:
            res = None
        else:
            # return as (data, indices, indptr) that can fed into scipy.sparse.csc_matrix
            res = (
                np.frombuffer(
                    ffi.buffer(cValsPtr[0], nNz[0] * 16), dtype=complex
                ).copy(),
                np.frombuffer(
                    ffi.buffer(RowIdxPtr[0], nNz[0] * 4), dtype=np.int32
                ).copy(),
                np.frombuffer(
                    ffi.buffer(ColPtr[0], (nBus[0] + 1) * 4), dtype=np.int32
                ).copy(),
            )
        lib.DSS_Dispose_PInteger(ColPtr)
        lib.DSS_Dispose_PInteger(RowIdxPtr)
        lib.DSS_Dispose_PDouble(cValsPtr)
        api_util._check_for_error()
        return res

    def ZeroInjCurr(self):
        self._lib.YMatrix_ZeroInjCurr()

    def GetSourceInjCurrents(self):
        self._lib.YMatrix_GetSourceInjCurrents()

    def GetPCInjCurr(self):
        self._lib.YMatrix_GetPCInjCurr()

    def BuildYMatrixD(self, BuildOps, AllocateVI):
        self._lib.YMatrix_BuildYMatrixD(BuildOps, AllocateVI)

    def AddInAuxCurrents(self, SType):
        self._lib.YMatrix_AddInAuxCurrents(SType)

    def IVector(self):
        """Get access to the internal Current pointer"""
        IvectorPtr = self._api_util.ffi.new("double**")
        self._lib.YMatrix_getIpointer(IvectorPtr)
        return IvectorPtr[0]

    def VVector(self):
        """Get access to the internal Voltage pointer"""
        VvectorPtr = self._api_util.ffi.new("double**")
        self._lib.YMatrix_getVpointer(VvectorPtr)
        return VvectorPtr[0]

    def SolveSystem(self, NodeV=None):
        if NodeV is not None and type(NodeV) is not np.ndarray:
            NodeV = np.array(NodeV)
        if NodeV is None:
            NodeVPtr = self._api_util.ffi.NULL
        else:
            NodeVPtr = self._api_util.ffi.cast("double *", NodeV.ctypes.data)

        return self._lib.YMatrix_SolveSystem(NodeVPtr)


    def SystemYChanged(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.YMatrix_Get_SystemYChanged()

        # Setter
        (value,) = args
        self._lib.YMatrix_Set_SystemYChanged(value)

    def UseAuxCurrents(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.YMatrix_Get_UseAuxCurrents()

        # Setter
        (value,) = args
        self._lib.YMatrix_Set_UseAuxCurrents(value)

    def SolverOptions(self, *args):
        """Sparse solver options. See the enumeration SparseSolverOptions"""
        # Getter
        if len(args) == 0:
            return self._lib.YMatrix_Get_SolverOptions()

        # Setter
        (Value,) = args
        self._lib.YMatrix_Set_SolverOptions(Value)

    def getI(self):
        """Get the data from the internal Current pointer"""
        IvectorPtr = self.IVector()
        return self._api_util.ffi.unpack(
            IvectorPtr, 2 * (self._lib.Circuit_Get_NumNodes() + 1)
        )

    def getV(self):
        """Get the data from the internal Voltage pointer"""
        VvectorPtr = self.VVector()
        return self._api_util.ffi.unpack(
            VvectorPtr, 2 * (self._lib.Circuit_Get_NumNodes() + 1)
        )

    def CheckConvergence(self):
        return self._lib.YMatrix_CheckConvergence()

    def SetGeneratordQdV(self):
        self._lib.YMatrix_SetGeneratordQdV()

    def LoadsNeedUpdating(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.YMatrix_Get_LoadsNeedUpdating()

        # Setter
        (value,) = args
        self._lib.YMatrix_Set_LoadsNeedUpdating(value)

    def SolutionInitialized(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.YMatrix_Get_SolutionInitialized()

        # Setter
        (value,) = args
        self._lib.YMatrix_Set_SolutionInitialized(value)

    def Iteration(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.YMatrix_Get_Iteration()

        # Setter
        (value,) = args
        self._lib.YMatrix_Set_Iteration(value)


_YMatrix = IYMatrix(api_util)

# For backwards compatibility, bind to the default instance
AddInAuxCurrents = _YMatrix.AddInAuxCurrents
BuildYMatrixD = _YMatrix.BuildYMatrixD
CheckConvergence = _YMatrix.CheckConvergence
GetPCInjCurr = _YMatrix.GetPCInjCurr
GetSourceInjCurrents = _YMatrix.GetSourceInjCurrents
IVector = _YMatrix.IVector
Iteration = _YMatrix.Iteration
LoadsNeedUpdating = _YMatrix.LoadsNeedUpdating
SetGeneratordQdV = _YMatrix.SetGeneratordQdV
SolutionInitialized = _YMatrix.SolutionInitialized
SolveSystem = _YMatrix.SolveSystem
SolverOptions = _YMatrix.SolverOptions
SystemYChanged = _YMatrix.SystemYChanged
UseAuxCurrents = _YMatrix.UseAuxCurrents
VVector = _YMatrix.VVector
ZeroInjCurr = _YMatrix.ZeroInjCurr
getI = _YMatrix.getI
getV = _YMatrix.getV
getYsparse = _YMatrix.getYsparse
_columns = _YMatrix._columns
__all__ = [
    "AddInAuxCurrents",
    "BuildYMatrixD",
    "CheckConvergence",
    "GetPCInjCurr",
    "GetSourceInjCurrents",
    "IVector",
    "Iteration",
    "LoadsNeedUpdating",
    "SetGeneratordQdV",
    "SolutionInitialized",
    "SolveSystem",
    "SolverOptions",
    "SystemYChanged",
    "UseAuxCurrents",
    "VVector",
    "ZeroInjCurr",
    "getI",
    "getV",
    "getYsparse",
]



from ._utils import api_util
from .Bases import Base


class IError(Base):
    __slots__ = []

    __name__ = "Error"
    _api_prefix = "Error"
    _columns = ["Description", "Number", "EarlyAbort"]

    def Description(self):
        """
        Description of error for last operation

        Original COM help: https://opendss.epri.com/Description1.html
        """
        return self._lib.Error_Get_Description()

    def Number(self):
        """
        Error Number (returns current value and then resets to zero)

        Original COM help: https://opendss.epri.com/Number.html
        """
        return self._lib.Error_Get_Number()

    def EarlyAbort(self, *args):
        """
        EarlyAbort controls whether all errors halts the DSS script processing (Compile/Redirect), defaults to True.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.Error_Get_EarlyAbort()

        # Setter
        (Value,) = args
        self._lib.Error_Set_EarlyAbort(Value)

    def ExtendedErrors(self, *args):
        """
        Controls whether the extended error mechanism is used. Defaults to True.

        Extended errors are errors derived from checks across the API to ensure
        a valid state. Although many of these checks are already present in the
        original/EPRI's COM interface, the checks do not produce any error
        message. An error value can be returned by a function but this value
        can, for many of the functions, be a valid value. As such, the user
        has no means to detect an invalid API call.

        Extended errors use the Error interface to provide a more clear message
        and should help users, especially new users, to find usage issues earlier.

        At Python level, an exception is raised when an error is detected through
        the Error interface.

        The current default state is ON. For compatibility, the user can turn it
        off to restore the previous behavior.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.Error_Get_ExtendedErrors()

        # Setter
        (Value,) = args
        self._lib.Error_Set_ExtendedErrors(Value)

    def UseExceptions(self, *args):
        """
        Controls whether the automatic error checking mechanism is enable, i.e., if
        the DSS engine errors (from the `Error` interface) are mapped exception when
        detected.

        **When disabled, the user takes responsibility for checking for errors.**
        This can be done through the `Error` interface. When `Error.Number` is not
        zero, there should be an error message in `Error.Description`. This is compatible
        with the behavior on EPRI's OpenDSS (Windows-only COM implementation) when
        `AllowForms` is disabled.

        Users can also use the DSS command `Export ErrorLog` to inspect for errors.

        With EPRI's OpenDSS engines, in contrast to our main AltDSS engine, users are
        also required to set `AllowForms` to `False`, otherwise the engine does not
        populate the Error.Number API and the error is consumed by the popup form or
        terminal message.

        **NOTE:** this used to be a global settings. Since OpenDSSDirect.py v0.10.0,
        it only affects the target instance.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.using_exceptions

        # Setter
        (value,) = args
        self._lib.using_exceptions = value


_Error = IError(api_util)

# For backwards compatibility, bind to the default instance
Description = _Error.Description
EarlyAbort = _Error.EarlyAbort
ExtendedErrors = _Error.ExtendedErrors
Number = _Error.Number
UseExceptions = _Error.UseExceptions
_columns = _Error._columns
__all__ = ["Description", "EarlyAbort", "ExtendedErrors", "Number", "UseExceptions"]



from ._utils import api_util
from .Bases import Iterable


class ILoadShape(Iterable):
    __slots__ = []

    __name__ = "LoadShape"
    _api_prefix = "LoadShapes"
    _columns = [
        "Name",
        "Idx",
        "UseActual",
        "Npts",
        "HrInterval",
        "MinInterval",
        "SInterval",
        "PBase",
        "PMult",
        "QBase",
        "QMult",
        "TimeArray",
    ]

    def New(self, Name):
        """Create a new LoadShape, with default parameters"""
        return self._lib.LoadShapes_New(Name)

    def Normalize(self):
        """Normalize the LoadShape data inplace"""
        self._lib.LoadShapes_Normalize()

    def HrInterval(self, *args):
        """
        Fixed interval time value, in hours.

        Original COM help: https://opendss.epri.com/HrInterval.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_HrInterval()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_HrInterval(Value)

    def MinInterval(self, *args):
        """
        Fixed Interval time value, in minutes

        Original COM help: https://opendss.epri.com/MinInterval.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_MinInterval()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_MinInterval(Value)

    def Npts(self, *args):
        """
        Get/set Number of points in active Loadshape.

        Original COM help: https://opendss.epri.com/Npts.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_Npts()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_Npts(Value)

    def PBase(self, *args):
        """
        Base P value for normalization. Default is zero, meaning the peak will be used.

        Original COM help: https://opendss.epri.com/Pbase.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_PBase()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_PBase(Value)

    def PMult(self, *args):
        """
        Array of doubles for the P multiplier in the Loadshape.

        Original COM help: https://opendss.epri.com/Pmult.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_Pmult_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LoadShapes_Set_Pmult(ValuePtr, ValueCount)

    def QBase(self, *args):
        """
        Base for normalizing Q curve. If left at zero, the peak value is used.

        Original COM help: https://opendss.epri.com/Qbase.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_Qbase()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_Qbase(Value)

    def QMult(self, *args):
        """
        Array of doubles containing the Q multipliers.

        Original COM help: https://opendss.epri.com/Qmult.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_Qmult_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LoadShapes_Set_Qmult(ValuePtr, ValueCount)

    def TimeArray(self, *args):
        """
        Time array in hours corresponding to P and Q multipliers when the Interval=0.

        Original COM help: https://opendss.epri.com/TimeArray.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_TimeArray_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LoadShapes_Set_TimeArray(ValuePtr, ValueCount)

    def UseActual(self, *args):
        """
        Boolean flag to let Loads know to use the actual value in the curve rather than use the value as a multiplier.

        Original COM help: https://opendss.epri.com/UseActual.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_UseActual()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_UseActual(Value)

    def SInterval(self, *args):
        """
        Fixed interval time value, in seconds.

        Original COM help: https://opendss.epri.com/Sinterval.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LoadShapes_Get_SInterval()

        # Setter
        (Value,) = args
        self._lib.LoadShapes_Set_SInterval(Value)

    def UseFloat32(self):
        """
        Converts the current LoadShape data to float32/single precision.
        If there is no data or the data is already represented using float32, nothing is done.

        **(API Extension)**
        """
        self._lib.LoadShapes_UseFloat32()

    def UseFloat64(self):
        """
        Converts the current LoadShape data to float64/double precision.
        If there is no data or the data is already represented using float64, nothing is done.

        **(API Extension)**
        """
        self._lib.LoadShapes_UseFloat64()


_LoadShape = ILoadShape(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _LoadShape.AllNames
Count = _LoadShape.Count
First = _LoadShape.First
HrInterval = _LoadShape.HrInterval
Idx = _LoadShape.Idx
MinInterval = _LoadShape.MinInterval
Name = _LoadShape.Name
New = _LoadShape.New
Next = _LoadShape.Next
Normalize = _LoadShape.Normalize
Npts = _LoadShape.Npts
PBase = _LoadShape.PBase
PMult = _LoadShape.PMult
QBase = _LoadShape.QBase
QMult = _LoadShape.QMult
SInterval = _LoadShape.SInterval
TimeArray = _LoadShape.TimeArray
UseActual = _LoadShape.UseActual
UseFloat32 = _LoadShape.UseFloat32
UseFloat64 = _LoadShape.UseFloat64
_columns = _LoadShape._columns
__all__ = [
    "AllNames",
    "Count",
    "First",
    "HrInterval",
    "Idx",
    "MinInterval",
    "Name",
    "New",
    "Next",
    "Normalize",
    "Npts",
    "PBase",
    "PMult",
    "QBase",
    "QMult",
    "SInterval",
    "TimeArray",
    "UseActual",
    "UseFloat32",
    "UseFloat64",
]



from ._utils import api_util
from .Bases import Base


class IProperties(Base):
    __slots__ = []
    __name__ = "Properties"
    _api_prefix = "DSSProperty"
    _columns = ["Description", "Name", "Value"]

    def Description(self):
        """
        Description of the property.

        Original COM help: https://opendss.epri.com/Description.html
        """
        return self._lib.DSSProperty_Get_Description()

    def Name(self):
        """
        Name of Property

        Original COM help: https://opendss.epri.com/Name6.html
        """
        return self._lib.DSSProperty_Get_Name()

    def _setCurrentProperty(self, argIndex_or_Name):
        """
        Sets the current DSS property based on a 1-based integer (or integer as
        a string) as an property index, or a string as a property name.
        """
        try:
            if not isinstance(argIndex_or_Name, int):
                argIndex_or_Name = int(argIndex_or_Name)

            # use 1-based index here for compatibility
            self._lib.DSSProperty_Set_Index(argIndex_or_Name - 1)

        except ValueError:
            # if we cannot convert to integer, use string as name instead
            if not isinstance(argIndex_or_Name, bytes):
                argIndex_or_Name = argIndex_or_Name.encode(self._api_util.codec)

            self._lib.DSSProperty_Set_Name(argIndex_or_Name)
        

    def Value(self, *args):
        # Getter
        if len(args) == 0:
            # General getter
            return self._lib.DSSProperty_Get_Val()
        elif len(args) == 1:
            # Getter by index as str or integer
            argIndex_or_Name, = args

            self._setCurrentProperty(argIndex_or_Name)
            return self._lib.DSSProperty_Get_Val()

        # Setter by index as strs
        argIndex_or_Name, Value = args
        if not isinstance(Value, bytes):
            Value = str(Value).encode(self._api_util.codec)

        self._setCurrentProperty(argIndex_or_Name)
        self._lib.DSSProperty_Set_Val(Value)


_Properties = IProperties(api_util)

# For backwards compatibility, bind to the default instance
Description = _Properties.Description
Name = _Properties.Name
Value = _Properties.Value
_columns = _Properties._columns
__all__ = ["Description", "Name", "Value"]



from ._utils import api_util
from .Bases import Base
from dss import ControlModes, SolutionAlgorithms, SolveModes


class ISolution(Base):
    __slots__ = []

    __name__ = "Solution"
    _api_prefix = "Solution"
    _columns = [
        "MinIterations",
        "MaxIterations",
        "MaxControlIterations",
        "TotalIterations",
        "ControlIterations",
        "MostIterationsDone",
        "Number",
        "ProcessTime",
        "AddType",
        "GenkW",
        "DblHour",
        "Capkvar",
        "Seconds",
        "GenMult",
        "DefaultYearly",
        "IntervalHrs",
        "Converged",
        "ModeID",
        "TimeTimeStep",
        "TotalTime",
        "LoadModel",
        "EventLog",
        "Iterations",
        "GenPF",
        "Frequency",
        "LoadMult",
        "Random",
        "PctGrowth",
        "Year",
        "Algorithm",
        "Hour",
        "Convergence",
        "ControlMode",
        "LDCurve",
        "StepSize",
        "DefaultDaily",
        "ControlActionsDone",
        "Mode",
        "SystemYChanged",
    ]

    def BuildYMatrix(self, BuildOption, AllocateVI):
        self._lib.Solution_BuildYMatrix(BuildOption, AllocateVI)

    def CheckControls(self):
        self._lib.Solution_CheckControls()

    def CheckFaultStatus(self):
        self._lib.Solution_CheckFaultStatus()

    def Cleanup(self):
        self._lib.Solution_Cleanup()

    def DoControlActions(self):
        self._lib.Solution_DoControlActions()

    def FinishTimeStep(self):
        self._lib.Solution_FinishTimeStep()

    def InitSnap(self):
        self._lib.Solution_InitSnap()

    def SampleControlDevices(self):
        self._lib.Solution_SampleControlDevices()

    def SampleDoControlActions(self):
        self._lib.Solution_Sample_DoControlActions()

    def Solve(self):
        self._lib.Solution_Solve()

    def SolveDirect(self):
        self._lib.Solution_SolveDirect()

    def SolveNoControl(self):
        self._lib.Solution_SolveNoControl()

    def SolvePFlow(self):
        self._lib.Solution_SolvePflow()

    def SolvePlusControl(self):
        self._lib.Solution_SolvePlusControl()

    def SolveSnap(self):
        self._lib.Solution_SolveSnap()

    def AddType(self, *args):
        """
        Type of device to add in AutoAdd Mode: {dssGen (Default) | dssCap}

        Original COM help: https://opendss.epri.com/AddType.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_AddType()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_AddType(Value)

    def Algorithm(self, *args):
        """
        Base Solution algorithm

        Original COM help: https://opendss.epri.com/Algorithm.html
        """
        # Getter
        if len(args) == 0:
            return SolutionAlgorithms(self._lib.Solution_Get_Algorithm())

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Algorithm(Value)

    def Capkvar(self, *args):
        """
        Capacitor kvar for adding capacitors in AutoAdd mode

        Original COM help: https://opendss.epri.com/Capkvar.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Capkvar()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Capkvar(Value)

    def ControlActionsDone(self, *args):
        """
        Flag indicating the control actions are done.

        Original COM help: https://opendss.epri.com/ControlActionsDone.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_ControlActionsDone()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_ControlActionsDone(Value)

    def ControlIterations(self, *args):
        """
        Value of the control iteration counter

        Original COM help: https://opendss.epri.com/ControlIterations.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_ControlIterations()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_ControlIterations(Value)

    def ControlMode(self, *args):
        """
        Modes for control devices

        Original COM help: https://opendss.epri.com/ControlMode.html
        """
        # Getter
        if len(args) == 0:
            return ControlModes(self._lib.Solution_Get_ControlMode())

        # Setter
        (Value,) = args
        self._lib.Solution_Set_ControlMode(Value)

    def Converged(self, *args):
        """
        Flag to indicate whether the circuit solution converged

        Original COM help: https://opendss.epri.com/Converged.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Converged()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Converged(Value)

    def DefaultDaily(self, *args):
        """
        Default daily load shape (defaults to "Default")

        Original COM help: https://opendss.epri.com/DefaultDaily.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_DefaultDaily()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_DefaultDaily(Value)

    def DefaultYearly(self, *args):
        """
        Default Yearly load shape (defaults to "Default")

        Original COM help: https://opendss.epri.com/DefaultYearly.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_DefaultYearly()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_DefaultYearly(Value)

    def EventLog(self):
        """
        Array of strings containing the Event Log

        Original COM help: https://opendss.epri.com/EventLog.html
        """
        return self._lib.Solution_Get_EventLog()

    def Frequency(self, *args):
        """
        Set the Frequency for next solution

        Original COM help: https://opendss.epri.com/Frequency1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Frequency()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Frequency(Value)

    def GenMult(self, *args):
        """
        Default Multiplier applied to generators (like LoadMult)

        Original COM help: https://opendss.epri.com/GenMult.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_GenMult()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_GenMult(Value)

    def GenPF(self, *args):
        """
        PF for generators in AutoAdd mode

        Original COM help: https://opendss.epri.com/GenPF.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_GenPF()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_GenPF(Value)

    def GenkW(self, *args):
        """
        Generator kW for AutoAdd mode

        Original COM help: https://opendss.epri.com/GenkW.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_GenkW()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_GenkW(Value)

    def Hour(self, *args):
        """
        Set Hour for time series solutions.

        Original COM help: https://opendss.epri.com/Hour.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Hour()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Hour(Value)

    def IntervalHrs(self, *args):
        """
        Get/Set the Solution.IntervalHrs variable used for devices that integrate / custom solution algorithms
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_IntervalHrs()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_IntervalHrs(Value)

    def Iterations(self):
        """
        Number of iterations taken for last solution. (Same as Totaliterations)

        Original COM help: https://opendss.epri.com/Iterations.html
        """
        return self._lib.Solution_Get_Iterations()

    def LDCurve(self, *args):
        """
        Load-Duration Curve name for LD modes

        Original COM help: https://opendss.epri.com/LDCurve.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_LDCurve()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_LDCurve(Value)

    def LoadModel(self, *args):
        """
        Load Model: {dssPowerFlow (default) | dssAdmittance}

        Original COM help: https://opendss.epri.com/LoadModel.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_LoadModel()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_LoadModel(Value)

    def LoadMult(self, *args):
        """
        Default load multiplier applied to all non-fixed loads

        Original COM help: https://opendss.epri.com/LoadMult.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_LoadMult()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_LoadMult(Value)

    def MaxControlIterations(self, *args):
        """
        Maximum allowable control iterations

        Original COM help: https://opendss.epri.com/MaxControlIterations.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_MaxControlIterations()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_MaxControlIterations(Value)

    def MaxIterations(self, *args):
        """
        Max allowable iterations.

        Original COM help: https://opendss.epri.com/MaxIterations.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_MaxIterations()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_MaxIterations(Value)

    def MinIterations(self, *args):
        """
        Minimum number of iterations required for a power flow solution.

        Original COM help: https://opendss.epri.com/MinIterations.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_MinIterations()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_MinIterations(Value)

    def Mode(self, *args):
        """
        Set present solution mode

        Original COM help: https://opendss.epri.com/Mode2.html
        """
        # Getter
        if len(args) == 0:
            return SolveModes(self._lib.Solution_Get_Mode())

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Mode(Value)

    def ModeID(self):
        """
        ID (text) of the present solution mode

        Original COM help: https://opendss.epri.com/ModeID.html
        """
        return self._lib.Solution_Get_ModeID()

    def MostIterationsDone(self):
        """
        Max number of iterations required to converge at any control iteration of the most recent solution.

        Original COM help: https://opendss.epri.com/MostIterationsDone.html
        """
        return self._lib.Solution_Get_MostIterationsDone()

    def Number(self, *args):
        """
        Number of solutions to perform for Monte Carlo and time series simulations

        Original COM help: https://opendss.epri.com/Number1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Number()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Number(Value)

    def ProcessTime(self):
        """
        Gets the time required to perform the latest solution (Read only)

        Original COM help: https://opendss.epri.com/Process_Time.html
        """
        return self._lib.Solution_Get_Process_Time()

    def Random(self, *args):
        """
        Randomization mode for random variables "Gaussian" or "Uniform"

        Original COM help: https://opendss.epri.com/Random.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Random()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Random(Value)

    def Seconds(self, *args):
        """
        Seconds from top of the hour.

        Original COM help: https://opendss.epri.com/Seconds.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Seconds()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Seconds(Value)

    def StepSize(self, *args):
        """
        Time step size in sec

        Original COM help: https://opendss.epri.com/StepSize.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_StepSize()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_StepSize(Value)

    def SystemYChanged(self):
        """
        Flag that indicates if elements of the System Y have been changed by recent activity.

        Original COM help: https://opendss.epri.com/SystemYChanged.html
        """
        return self._lib.Solution_Get_SystemYChanged()

    def TimeTimeStep(self):
        """
        Get the solution process time + sample time for time step

        Original COM help: https://opendss.epri.com/Time_of_Step.html
        """
        return self._lib.Solution_Get_Time_of_Step()

    def Convergence(self, *args):
        """
        Solution convergence tolerance.

        Original COM help: https://opendss.epri.com/Tolerance.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Tolerance()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Tolerance(Value)

    def TotalTime(self, *args):
        """
        Gets/sets the accumulated time of the simulation

        This accumulator has to be reset manually.

        Original COM help: https://opendss.epri.com/Total_Time.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Total_Time()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Total_Time(Value)

    def TotalIterations(self):
        """
        Total iterations including control iterations for most recent solution.

        Original COM help: https://opendss.epri.com/Totaliterations.html
        """
        return self._lib.Solution_Get_Totaliterations()

    def Year(self, *args):
        """
        Set year for planning studies

        Original COM help: https://opendss.epri.com/Year.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_Year()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_Year(Value)

    def DblHour(self, *args):
        """
        Hour as a double, including fractional part

        Original COM help: https://opendss.epri.com/dblHour1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_dblHour()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_dblHour(Value)

    def PctGrowth(self, *args):
        """
        Percent default  annual load growth rate

        Original COM help: https://opendss.epri.com/pctGrowth.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Solution_Get_pctGrowth()

        # Setter
        (Value,) = args
        self._lib.Solution_Set_pctGrowth(Value)

    def StepSizeHr(self, Value):
        """(write-only) Set Stepsize in Hr"""
        self._lib.Solution_Set_StepsizeHr(Value)

    def StepSizeMin(self, Value):
        """(write-only) Set Stepsize in minutes"""
        self._lib.Solution_Set_StepsizeMin(Value)

    def BusLevels(self):
        """
        Bus levels for all the buses in the model.

        The bus levels are calculated after calculating the incidence branch-to-node (B2N)
        matrix and they represent the distance from the buses to a reference that goes from
        the feeder head to the farthest bus in the model. The bus level index matches with
        the bus list obtained with the circuit interface.

        Original COM help: https://opendss.epri.com/BusLevels.html
        """
        return self._lib.Solution_Get_BusLevels_GR()

    def IncMatrix(self):
        """
        Incidence branch-to-node (B2N) matrix calculated for the model as a vector of integers.

        The vector represents a sparse matrix (non-zero values are the only ones delivered) and
        can be interpreted as follows: The first element is the row number, the second one is
        the column and the third is the value, this way, by dividing the number of elements
        in the array by 3 the user can obtain the number of rows in case of wanting to sort
        the vector values within a matrix.

        Original COM help: https://opendss.epri.com/IncMatrix.html
        """
        return self._lib.Solution_Get_IncMatrix_GR()

    def IncMatrixCols(self):
        """
        Names of the columns of the branch-to-node (B2N) matrix.

        Original COM help: https://opendss.epri.com/IncMatrixCols.html
        """
        return self._lib.Solution_Get_IncMatrixCols()

    def IncMatrixRows(self):
        """
        Names of the rows of the branch-to-node (B2N) matrix.

        Original COM help: https://opendss.epri.com/IncMatrixRows.html
        """
        return self._lib.Solution_Get_IncMatrixRows()

    def Laplacian(self):
        """
        Laplacian matrix calculated in OpenDSS based on the latest branch-to-node (B2N) matrix.

        The vector represents a sparse matrix (non-zero values are the only ones delivered) and
        can be interpreted as follows: The first element is the row number, the second one is
        the column and the third is the value, this way, by dividing the number of elements
        in the array by 3 the user can obtain the number of rows in case of wanting to sort
        the vector values within a matrix. The tables for the columns and rows are the same
        as the columns for the B2N columns (square matrix).

        Original COM help: https://opendss.epri.com/Laplacian.html
        """
        return self._lib.Solution_Get_Laplacian_GR()


_Solution = ISolution(api_util)

# For backwards compatibility, bind to the default instance
AddType = _Solution.AddType
Algorithm = _Solution.Algorithm
BuildYMatrix = _Solution.BuildYMatrix
BusLevels = _Solution.BusLevels
Capkvar = _Solution.Capkvar
CheckControls = _Solution.CheckControls
CheckFaultStatus = _Solution.CheckFaultStatus
Cleanup = _Solution.Cleanup
ControlActionsDone = _Solution.ControlActionsDone
ControlIterations = _Solution.ControlIterations
ControlMode = _Solution.ControlMode
Converged = _Solution.Converged
Convergence = _Solution.Convergence
DblHour = _Solution.DblHour
DefaultDaily = _Solution.DefaultDaily
DefaultYearly = _Solution.DefaultYearly
DoControlActions = _Solution.DoControlActions
EventLog = _Solution.EventLog
FinishTimeStep = _Solution.FinishTimeStep
Frequency = _Solution.Frequency
GenMult = _Solution.GenMult
GenPF = _Solution.GenPF
GenkW = _Solution.GenkW
Hour = _Solution.Hour
IncMatrix = _Solution.IncMatrix
IncMatrixCols = _Solution.IncMatrixCols
IncMatrixRows = _Solution.IncMatrixRows
InitSnap = _Solution.InitSnap
IntervalHrs = _Solution.IntervalHrs
Iterations = _Solution.Iterations
LDCurve = _Solution.LDCurve
Laplacian = _Solution.Laplacian
LoadModel = _Solution.LoadModel
LoadMult = _Solution.LoadMult
MaxControlIterations = _Solution.MaxControlIterations
MaxIterations = _Solution.MaxIterations
MinIterations = _Solution.MinIterations
Mode = _Solution.Mode
ModeID = _Solution.ModeID
MostIterationsDone = _Solution.MostIterationsDone
Number = _Solution.Number
PctGrowth = _Solution.PctGrowth
ProcessTime = _Solution.ProcessTime
Random = _Solution.Random
SampleControlDevices = _Solution.SampleControlDevices
SampleDoControlActions = _Solution.SampleDoControlActions
Seconds = _Solution.Seconds
Solve = _Solution.Solve
SolveDirect = _Solution.SolveDirect
SolveNoControl = _Solution.SolveNoControl
SolvePFlow = _Solution.SolvePFlow
SolvePlusControl = _Solution.SolvePlusControl
SolveSnap = _Solution.SolveSnap
StepSize = _Solution.StepSize
StepSizeHr = _Solution.StepSizeHr
StepSizeMin = _Solution.StepSizeMin
SystemYChanged = _Solution.SystemYChanged
TimeTimeStep = _Solution.TimeTimeStep
TotalIterations = _Solution.TotalIterations
TotalTime = _Solution.TotalTime
Year = _Solution.Year
_columns = _Solution._columns
__all__ = [
    "AddType",
    "Algorithm",
    "BuildYMatrix",
    "BusLevels",
    "Capkvar",
    "CheckControls",
    "CheckFaultStatus",
    "Cleanup",
    "ControlActionsDone",
    "ControlIterations",
    "ControlMode",
    "Converged",
    "Convergence",
    "DblHour",
    "DefaultDaily",
    "DefaultYearly",
    "DoControlActions",
    "EventLog",
    "FinishTimeStep",
    "Frequency",
    "GenMult",
    "GenPF",
    "GenkW",
    "Hour",
    "IncMatrix",
    "IncMatrixCols",
    "IncMatrixRows",
    "InitSnap",
    "IntervalHrs",
    "Iterations",
    "LDCurve",
    "Laplacian",
    "LoadModel",
    "LoadMult",
    "MaxControlIterations",
    "MaxIterations",
    "MinIterations",
    "Mode",
    "ModeID",
    "MostIterationsDone",
    "Number",
    "PctGrowth",
    "ProcessTime",
    "Random",
    "SampleControlDevices",
    "SampleDoControlActions",
    "Seconds",
    "Solve",
    "SolveDirect",
    "SolveNoControl",
    "SolvePFlow",
    "SolvePlusControl",
    "SolveSnap",
    "StepSize",
    "StepSizeHr",
    "StepSizeMin",
    "SystemYChanged",
    "TimeTimeStep",
    "TotalIterations",
    "TotalTime",
    "Year",
]



from ._utils import api_util
from .Bases import Iterable


class IRegControls(Iterable):
    __slots__ = []

    __name__ = "RegControls"
    _api_prefix = "RegControls"
    _columns = [
        "Name",
        "Idx",
        "Transformer",
        "Winding",
        "MonitoredBus",
        "CTPrimary",
        "PTRatio",
        "Delay",
        "IsInverseTime",
        "IsReversible",
        "MaxTapChange",
        "TapDelay",
        "TapNumber",
        "TapWinding",
        "VoltageLimit",
        "ForwardBand",
        "ForwardR",
        "ForwardX",
        "ForwardVreg",
        "ReverseBand",
        "ReverseR",
        "ReverseX",
        "ReverseVreg",
    ]

    def Reset(self):
        self._lib.RegControls_Reset()

    def CTPrimary(self, *args):
        """
        CT primary ampere rating (secondary is 0.2 amperes)

        Original COM help: https://opendss.epri.com/CTPrimary.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_CTPrimary()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_CTPrimary(Value)

    def Delay(self, *args):
        """
        Time delay [s] after arming before the first tap change. Control may reset before actually changing taps.

        Original COM help: https://opendss.epri.com/Delay2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_Delay()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_Delay(Value)

    def ForwardBand(self, *args):
        """
        Regulation bandwidth in forward direction, centered on Vreg

        Original COM help: https://opendss.epri.com/ForwardBand.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ForwardBand()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ForwardBand(Value)

    def ForwardR(self, *args):
        """
        LDC R setting in Volts

        Original COM help: https://opendss.epri.com/ForwardR.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ForwardR()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ForwardR(Value)

    def ForwardVreg(self, *args):
        """
        Target voltage in the forward direction, on PT secondary base.

        Original COM help: https://opendss.epri.com/ForwardVreg.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ForwardVreg()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ForwardVreg(Value)

    def ForwardX(self, *args):
        """
        LDC X setting in Volts

        Original COM help: https://opendss.epri.com/ForwardX.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ForwardX()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ForwardX(Value)

    def IsInverseTime(self, *args):
        """
        Time delay is inversely adjusted, proportional to the amount of voltage outside the regulating band.

        Original COM help: https://opendss.epri.com/IsInverseTime.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_IsInverseTime()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_IsInverseTime(Value)

    def IsReversible(self, *args):
        """
        Regulator can use different settings in the reverse direction.  Usually not applicable to substation transformers.

        Original COM help: https://opendss.epri.com/IsReversible.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_IsReversible()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_IsReversible(Value)

    def MaxTapChange(self, *args):
        """
        Maximum tap change per iteration in STATIC solution mode. 1 is more realistic, 16 is the default for a faster solution.

        Original COM help: https://opendss.epri.com/MaxTapChange.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_MaxTapChange()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_MaxTapChange(Value)

    def MonitoredBus(self, *args):
        """
        Name of a remote regulated bus, in lieu of LDC settings

        Original COM help: https://opendss.epri.com/MonitoredBus.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_MonitoredBus()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_MonitoredBus(Value)

    def PTRatio(self, *args):
        """
        PT ratio for voltage control settings

        Original COM help: https://opendss.epri.com/PTratio1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_PTratio()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_PTratio(Value)

    def ReverseBand(self, *args):
        """
        Bandwidth in reverse direction, centered on reverse Vreg.

        Original COM help: https://opendss.epri.com/ReverseBand.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ReverseBand()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ReverseBand(Value)

    def ReverseR(self, *args):
        """
        Reverse LDC R setting in Volts.

        Original COM help: https://opendss.epri.com/ReverseR.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ReverseR()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ReverseR(Value)

    def ReverseVreg(self, *args):
        """
        Target voltage in the revese direction, on PT secondary base.

        Original COM help: https://opendss.epri.com/ReverseVreg.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ReverseVreg()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ReverseVreg(Value)

    def ReverseX(self, *args):
        """
        Reverse LDC X setting in volts.

        Original COM help: https://opendss.epri.com/ReverseX.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_ReverseX()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_ReverseX(Value)

    def TapDelay(self, *args):
        """
        Time delay [s] for subsequent tap changes in a set. Control may reset before actually changing taps.

        Original COM help: https://opendss.epri.com/TapDelay.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_TapDelay()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_TapDelay(Value)

    def TapNumber(self, *args):
        """
        Integer number of the tap that the controlled transformer winding is currently on.

        Original COM help: https://opendss.epri.com/TapNumber.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_TapNumber()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_TapNumber(Value)

    def TapWinding(self, *args):
        """
        Tapped winding number

        Original COM help: https://opendss.epri.com/TapWinding.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_TapWinding()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_TapWinding(Value)

    def Transformer(self, *args):
        """
        Name of the transformer this regulator controls

        Original COM help: https://opendss.epri.com/Transformer.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_Transformer()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_Transformer(Value)

    def VoltageLimit(self, *args):
        """
        First house voltage limit on PT secondary base.  Setting to 0 disables this function.

        Original COM help: https://opendss.epri.com/VoltageLimit.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_VoltageLimit()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_VoltageLimit(Value)

    def Winding(self, *args):
        """
        Winding number for PT and CT connections

        Original COM help: https://opendss.epri.com/Winding.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.RegControls_Get_Winding()

        # Setter
        (Value,) = args
        self._lib.RegControls_Set_Winding(Value)


_RegControls = IRegControls(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _RegControls.AllNames
CTPrimary = _RegControls.CTPrimary
Count = _RegControls.Count
Delay = _RegControls.Delay
First = _RegControls.First
ForwardBand = _RegControls.ForwardBand
ForwardR = _RegControls.ForwardR
ForwardVreg = _RegControls.ForwardVreg
ForwardX = _RegControls.ForwardX
Idx = _RegControls.Idx
IsInverseTime = _RegControls.IsInverseTime
IsReversible = _RegControls.IsReversible
MaxTapChange = _RegControls.MaxTapChange
MonitoredBus = _RegControls.MonitoredBus
Name = _RegControls.Name
Next = _RegControls.Next
PTRatio = _RegControls.PTRatio
Reset = _RegControls.Reset
ReverseBand = _RegControls.ReverseBand
ReverseR = _RegControls.ReverseR
ReverseVreg = _RegControls.ReverseVreg
ReverseX = _RegControls.ReverseX
TapDelay = _RegControls.TapDelay
TapNumber = _RegControls.TapNumber
TapWinding = _RegControls.TapWinding
Transformer = _RegControls.Transformer
VoltageLimit = _RegControls.VoltageLimit
Winding = _RegControls.Winding
_columns = _RegControls._columns
__all__ = [
    "AllNames",
    "CTPrimary",
    "Count",
    "Delay",
    "First",
    "ForwardBand",
    "ForwardR",
    "ForwardVreg",
    "ForwardX",
    "Idx",
    "IsInverseTime",
    "IsReversible",
    "MaxTapChange",
    "MonitoredBus",
    "Name",
    "Next",
    "PTRatio",
    "Reset",
    "ReverseBand",
    "ReverseR",
    "ReverseVreg",
    "ReverseX",
    "TapDelay",
    "TapNumber",
    "TapWinding",
    "Transformer",
    "VoltageLimit",
    "Winding",
]



from ._utils import api_util
from .Bases import Iterable


class IFuses(Iterable):
    __slots__ = []

    __name__ = "Fuses"
    _api_prefix = "Fuses"
    _columns = [
        "Name",
        "Idx",
        "NumPhases",
        "MonitoredObj",
        "MonitoredTerm",
        "Delay",
        "IsBlown",
        "TCCCurve",
        "RatedCurrent",
        "SwitchedObj",
        "SwitchedTerm",
        "State",
        "NormalState",
    ]

    def Close(self):
        """
        Close all phases of the fuse.

        Original COM help: https://opendss.epri.com/Close3.html
        """
        self._lib.Fuses_Close()

    def IsBlown(self):
        """
        Current state of the fuses. TRUE if any fuse on any phase is blown. Else FALSE.

        Original COM help: https://opendss.epri.com/IsBlown.html
        """
        return self._lib.Fuses_IsBlown()

    def Open(self):
        """
        Manual opening of all phases of the fuse.

        Original COM help: https://opendss.epri.com/Open2.html
        """
        self._lib.Fuses_Open()

    def Reset(self):
        """
        Reset fuse to normal state.

        Original COM help: https://opendss.epri.com/Reset7.html
        """
        self._lib.Fuses_Reset()

    def Delay(self, *args):
        """
        A fixed delay time in seconds added to the fuse blowing time determined by the TCC curve. Default is 0.
        This represents a fuse clear or other delay.

        Original COM help: https://opendss.epri.com/Delay1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_Delay()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_Delay(Value)

    def MonitoredObj(self, *args):
        """
        Full name of the circuit element to which the fuse is connected.

        Original COM help: https://opendss.epri.com/MonitoredObj1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_MonitoredObj()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_MonitoredObj(Value)

    def MonitoredTerm(self, *args):
        """
        Terminal number to which the fuse is connected.

        Original COM help: https://opendss.epri.com/MonitoredTerm1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_MonitoredTerm()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_MonitoredTerm(Value)

    def NumPhases(self):
        """
        Number of phases, this fuse.

        Original COM help: https://opendss.epri.com/NumPhases1.html
        """
        return self._lib.Fuses_Get_NumPhases()

    def RatedCurrent(self, *args):
        """
        Multiplier or actual amps for the TCCcurve object. Defaults to 1.0.

        Multiply current values of TCC curve by this to get actual amps.

        Original COM help: https://opendss.epri.com/RatedCurrent.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_RatedCurrent()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_RatedCurrent(Value)

    def SwitchedObj(self, *args):
        """
        Full name of the circuit element switch that the fuse controls.
        Defaults to the MonitoredObj.

        Original COM help: https://opendss.epri.com/SwitchedObj.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_SwitchedObj()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_SwitchedObj(Value)

    def SwitchedTerm(self, *args):
        """
        Number of the terminal of the controlled element containing the switch controlled by the fuse.

        Original COM help: https://opendss.epri.com/SwitchedTerm.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_SwitchedTerm()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_SwitchedTerm(Value)

    def TCCCurve(self, *args):
        """
        Name of the TCCcurve object that determines fuse blowing.

        Original COM help: https://opendss.epri.com/TCCcurve.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_TCCcurve()

        # Setter
        (Value,) = args
        self._lib.Fuses_Set_TCCcurve(Value)

    def State(self, *args):
        """
        Array of strings indicating the state of each phase of the fuse.

        Original COM help: https://opendss.epri.com/State2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_State()

        # Setter
        (Value,) = args
        self._set_string_array(self._lib.Fuses_Set_State, Value)

    def NormalState(self, *args):
        """
        Array of strings indicating the normal state of each phase of the fuse.

        Original COM help: https://opendss.epri.com/NormalState2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Fuses_Get_NormalState()

        # Setter
        (Value,) = args
        self._set_string_array(self._lib.Fuses_Set_NormalState, Value)


_Fuses = IFuses(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Fuses.AllNames
Close = _Fuses.Close
Count = _Fuses.Count
Delay = _Fuses.Delay
First = _Fuses.First
Idx = _Fuses.Idx
IsBlown = _Fuses.IsBlown
MonitoredObj = _Fuses.MonitoredObj
MonitoredTerm = _Fuses.MonitoredTerm
Name = _Fuses.Name
Next = _Fuses.Next
NormalState = _Fuses.NormalState
NumPhases = _Fuses.NumPhases
Open = _Fuses.Open
RatedCurrent = _Fuses.RatedCurrent
Reset = _Fuses.Reset
State = _Fuses.State
SwitchedObj = _Fuses.SwitchedObj
SwitchedTerm = _Fuses.SwitchedTerm
TCCCurve = _Fuses.TCCCurve
_columns = _Fuses._columns
__all__ = [
    "AllNames",
    "Close",
    "Count",
    "Delay",
    "First",
    "Idx",
    "IsBlown",
    "MonitoredObj",
    "MonitoredTerm",
    "Name",
    "Next",
    "NormalState",
    "NumPhases",
    "Open",
    "RatedCurrent",
    "Reset",
    "State",
    "SwitchedObj",
    "SwitchedTerm",
    "TCCCurve",
]



from ._utils import api_util
from .Bases import Iterable
from dss import LineUnits


class ILineCodes(Iterable):
    __slots__ = []

    __name__ = "LineCodes"
    _api_prefix = "LineCodes"
    _columns = [
        "Name",
        "Idx",
        "Phases",
        "Units",
        "IsZ1Z0",
        "R0",
        "R1",
        "X0",
        "X1",
        "C0",
        "C1",
        "Rmatrix",
        "Xmatrix",
        "Cmatrix",
        "EmergAmps",
        "NormAmps",
    ]

    def C0(self, *args):
        """
        Zero-sequence capacitance, nF per unit length

        Original COM help: https://opendss.epri.com/C2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_C0()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_C0(Value)

    def C1(self, *args):
        """
        Positive-sequence capacitance, nF per unit length

        Original COM help: https://opendss.epri.com/C3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_C1()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_C1(Value)

    def Cmatrix(self, *args):
        """
        Capacitance matrix, nF per unit length

        Original COM help: https://opendss.epri.com/Cmatrix1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_Cmatrix_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineCodes_Set_Cmatrix(ValuePtr, ValueCount)

    def EmergAmps(self, *args):
        """
        Emergency ampere rating

        Original COM help: https://opendss.epri.com/EmergAmps2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_EmergAmps()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_EmergAmps(Value)

    def IsZ1Z0(self):
        """
        Flag denoting whether impedance data were entered in symmetrical components

        Original COM help: https://opendss.epri.com/IsZ1Z0.html
        """
        return self._lib.LineCodes_Get_IsZ1Z0()

    def NormAmps(self, *args):
        """
        Normal Ampere rating

        Original COM help: https://opendss.epri.com/NormAmps1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_NormAmps()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_NormAmps(Value)

    def Phases(self, *args):
        """
        Number of Phases

        Original COM help: https://opendss.epri.com/Phases2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_Phases(Value)

    def R0(self, *args):
        """
        Zero-Sequence Resistance, ohms per unit length

        Original COM help: https://opendss.epri.com/R2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_R0()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_R0(Value)

    def R1(self, *args):
        """
        Positive-sequence resistance ohms per unit length

        Original COM help: https://opendss.epri.com/R3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_R1()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_R1(Value)

    def Rmatrix(self, *args):
        """
        Resistance matrix, ohms per unit length

        Original COM help: https://opendss.epri.com/Rmatrix1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_Rmatrix_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineCodes_Set_Rmatrix(ValuePtr, ValueCount)

    def Units(self, *args):
        # Getter
        if len(args) == 0:
            return LineUnits(self._lib.LineCodes_Get_Units())

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_Units(Value)

    def X0(self, *args):
        """
        Zero Sequence Reactance, Ohms per unit length

        Original COM help: https://opendss.epri.com/X2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_X0()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_X0(Value)

    def X1(self, *args):
        """
        Positive-sequence reactance, ohms per unit length

        Original COM help: https://opendss.epri.com/X3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_X1()

        # Setter
        (Value,) = args
        self._lib.LineCodes_Set_X1(Value)

    def Xmatrix(self, *args):
        """
        Reactance matrix, ohms per unit length

        Original COM help: https://opendss.epri.com/Xmatrix1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.LineCodes_Get_Xmatrix_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineCodes_Set_Xmatrix(ValuePtr, ValueCount)


_LineCodes = ILineCodes(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _LineCodes.AllNames
C0 = _LineCodes.C0
C1 = _LineCodes.C1
Cmatrix = _LineCodes.Cmatrix
Count = _LineCodes.Count
EmergAmps = _LineCodes.EmergAmps
First = _LineCodes.First
Idx = _LineCodes.Idx
IsZ1Z0 = _LineCodes.IsZ1Z0
Name = _LineCodes.Name
Next = _LineCodes.Next
NormAmps = _LineCodes.NormAmps
Phases = _LineCodes.Phases
R0 = _LineCodes.R0
R1 = _LineCodes.R1
Rmatrix = _LineCodes.Rmatrix
Units = _LineCodes.Units
X0 = _LineCodes.X0
X1 = _LineCodes.X1
Xmatrix = _LineCodes.Xmatrix
_columns = _LineCodes._columns
__all__ = [
    "AllNames",
    "C0",
    "C1",
    "Cmatrix",
    "Count",
    "EmergAmps",
    "First",
    "Idx",
    "IsZ1Z0",
    "Name",
    "Next",
    "NormAmps",
    "Phases",
    "R0",
    "R1",
    "Rmatrix",
    "Units",
    "X0",
    "X1",
    "Xmatrix",
]



from ._utils import api_util
from .Bases import Base


class IExecutive(Base):
    __slots__ = []

    __name__ = "Executive"
    _api_prefix = "DSS_Executive"
    _columns = ["NumCommands", "NumOptions"]

    def Command(self, i):
        """
        Get i-th command

        Original COM help: https://opendss.epri.com/Command.html
        """
        return self._lib.DSS_Executive_Get_Command(i)

    def CommandHelp(self, i):
        """
        Get help string for i-th command

        Original COM help: https://opendss.epri.com/CommandHelp.html
        """
        return self._lib.DSS_Executive_Get_CommandHelp(i)

    def Option(self, i):
        """
        Get i-th option

        Original COM help: https://opendss.epri.com/Option.html
        """
        return self._lib.DSS_Executive_Get_Option(i)

    def OptionHelp(self, i):
        """
        Get help string for i-th option

        Original COM help: https://opendss.epri.com/OptionHelp.html
        """
        return self._lib.DSS_Executive_Get_OptionHelp(i)

    def OptionValue(self, i):
        """
        Get present value of i-th option

        Original COM help: https://opendss.epri.com/OptionValue.html
        """
        return self._lib.DSS_Executive_Get_OptionValue(i)

    def NumCommands(self):
        """
        Number of DSS Executive Commands

        Original COM help: https://opendss.epri.com/NumCommands.html
        """
        return self._lib.DSS_Executive_Get_NumCommands()

    def NumOptions(self):
        """
        Number of DSS Executive Options

        Original COM help: https://opendss.epri.com/NumOptions.html
        """
        return self._lib.DSS_Executive_Get_NumOptions()


_Executive = IExecutive(api_util)

# For backwards compatibility, bind to the default instance
Command = _Executive.Command
CommandHelp = _Executive.CommandHelp
NumCommands = _Executive.NumCommands
NumOptions = _Executive.NumOptions
Option = _Executive.Option
OptionHelp = _Executive.OptionHelp
OptionValue = _Executive.OptionValue
_columns = _Executive._columns
__all__ = [
    "Command",
    "CommandHelp",
    "NumCommands",
    "NumOptions",
    "Option",
    "OptionHelp",
    "OptionValue",
]



from ._utils import api_util
from .Bases import Iterable


class ISensors(Iterable):
    __slots__ = []

    __name__ = "Sensors"
    _api_prefix = "Sensors"
    _columns = [
        "Name",
        "Idx",
        "MeteredElement",
        "MeteredTerminal",
        "IsDelta",
        "ReverseDelta",
        "Currents",
        "PctError",
        "Weight",
        "kVBase",
        "kvar",
        "kVS",
        "kW",
        "AllocationFactor",
    ]

    def Reset(self):
        self._lib.Sensors_Reset()

    def ResetAll(self):
        self._lib.Sensors_ResetAll()

    def Currents(self, *args):
        """
        Array of doubles for the line current measurements; don't use with kWS and kVARS.

        Original COM help: https://opendss.epri.com/Currents2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_Currents_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Sensors_Set_Currents(ValuePtr, ValueCount)

    def IsDelta(self, *args):
        """
        True if measured voltages are line-line. Currents are always line currents.

        Original COM help: https://opendss.epri.com/IsDelta2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_IsDelta()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_IsDelta(Value)

    def MeteredElement(self, *args):
        """
        Full Name of the measured element

        Original COM help: https://opendss.epri.com/MeteredElement1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_MeteredElement()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_MeteredElement(Value)

    def MeteredTerminal(self, *args):
        """
        Number of the measured terminal in the measured element.

        Original COM help: https://opendss.epri.com/MeteredTerminal1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_MeteredTerminal()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_MeteredTerminal(Value)

    def PctError(self, *args):
        """
        Assumed percent error in the Sensor measurement. Default is 1.

        Original COM help: https://opendss.epri.com/PctError.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_PctError()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_PctError(Value)

    def ReverseDelta(self, *args):
        """
        True if voltage measurements are 1-3, 3-2, 2-1.

        Original COM help: https://opendss.epri.com/ReverseDelta.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_ReverseDelta()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_ReverseDelta(Value)

    def Weight(self, *args):
        """
        Weighting factor for this Sensor measurement with respect to other Sensors. Default is 1.

        Original COM help: https://opendss.epri.com/Weight.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_Weight()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_Weight(Value)

    def kvar(self, *args):
        """
        Array of doubles for Q measurements. Overwrites Currents with a new estimate using kWS.

        Original COM help: https://opendss.epri.com/kVARS.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_kVARS_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Sensors_Set_kVARS(ValuePtr, ValueCount)

    def kVS(self, *args):
        """
        Array of doubles for the LL or LN (depending on Delta connection) voltage measurements.

        Original COM help: https://opendss.epri.com/kVS.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_kVS_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Sensors_Set_kVS(ValuePtr, ValueCount)

    def kVBase(self, *args):
        """
        Voltage base for the sensor measurements. LL for 2 and 3-phase sensors, LN for 1-phase sensors.

        Original COM help: https://opendss.epri.com/kVBase1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_kVbase()

        # Setter
        (Value,) = args
        self._lib.Sensors_Set_kVbase(Value)

    def kW(self, *args):
        """
        Array of doubles for P measurements. Overwrites Currents with a new estimate using kVARS.

        Original COM help: https://opendss.epri.com/kWS.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Sensors_Get_kWS_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Sensors_Set_kWS(ValuePtr, ValueCount)

    def AllocationFactor(self):
        """
        Array of doubles for the allocation factors for each phase.

        Original COM help: https://opendss.epri.com/AllocationFactor1.html
        """
        return self._lib.Sensors_Get_AllocationFactor_GR()


_Sensors = ISensors(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Sensors.AllNames
AllocationFactor = _Sensors.AllocationFactor
Count = _Sensors.Count
Currents = _Sensors.Currents
First = _Sensors.First
Idx = _Sensors.Idx
IsDelta = _Sensors.IsDelta
MeteredElement = _Sensors.MeteredElement
MeteredTerminal = _Sensors.MeteredTerminal
Name = _Sensors.Name
Next = _Sensors.Next
PctError = _Sensors.PctError
Reset = _Sensors.Reset
ResetAll = _Sensors.ResetAll
ReverseDelta = _Sensors.ReverseDelta
Weight = _Sensors.Weight
kVBase = _Sensors.kVBase
kVS = _Sensors.kVS
kW = _Sensors.kW
kvar = _Sensors.kvar
_columns = _Sensors._columns
__all__ = [
    "AllNames",
    "AllocationFactor",
    "Count",
    "Currents",
    "First",
    "Idx",
    "IsDelta",
    "MeteredElement",
    "MeteredTerminal",
    "Name",
    "Next",
    "PctError",
    "Reset",
    "ResetAll",
    "ReverseDelta",
    "Weight",
    "kVBase",
    "kVS",
    "kW",
    "kvar",
]



from ._utils import api_util
from .Bases import Iterable
from dss_python_backend.enums import OCPDevType as OCPDevTypeEnum

class IMeters(Iterable):
    __slots__ = []

    __name__ = "Meters"
    _api_prefix = "Meters"
    _columns = [
        "Name",
        "MeteredTerminal",
        "SeqListSize",
        "RegisterNames",
        "MeteredElement",
        "PeakCurrent",
        "AllocFactors",
        "AllEndElements",
        "SAIFIkW",
        "CountEndElements",
        "SAIDI",
        "TotalCustomers",
        "RegisterValues",
        "SAIFI",
        "CustInterrupts",
        "CountBranches",
        "CalcCurrent",
        "AllBranchesInZone",
        "NumSections",
        # "NumSectionCustomers",
        # "SectSeqidx",
        # "SumBranchFltRates",
        # "AvgRepairTime",
        # "SectTotalCust",
        # "OCPDeviceType",
        # "FaultRateXRepairHrs",
        # "NumSectionBranches",
    ]

    def CloseAllDIFiles(self):
        """
        Close All Demand Interval Files. Users are required to close the DI files at the end of a run.

        Original COM help: https://opendss.epri.com/CloseAllDIFiles.html
        """
        self._lib.Meters_CloseAllDIFiles()

    def DoReliabilityCalc(self, AssumeRestoration):
        """
        Calculate reliability indices

        Original COM help: https://opendss.epri.com/DoReliabilityCalc.html
        """
        self._lib.Meters_DoReliabilityCalc(AssumeRestoration)

    def OpenAllDIFiles(self):
        """
        Open Demand Interval (DI) files

        Original COM help: https://opendss.epri.com/OpenAllDIFiles.html
        """
        self._lib.Meters_OpenAllDIFiles()

    def Reset(self):
        """
        Resets registers of active meter.

        Original COM help: https://opendss.epri.com/Reset2.html
        """
        self._lib.Meters_Reset()

    def ResetAll(self):
        """
        Resets registers of all meter objects.

        Original COM help: https://opendss.epri.com/ResetAll.html
        """
        self._lib.Meters_ResetAll()

    def Sample(self):
        """
        Forces active Meter to take a sample.

        Original COM help: https://opendss.epri.com/Sample1.html
        """
        self._lib.Meters_Sample()

    def SampleAll(self):
        """
        Causes all EnergyMeter objects to take a sample at the present time.

        Original COM help: https://opendss.epri.com/SampleAll.html
        """
        self._lib.Meters_SampleAll()

    def Save(self):
        """
        Saves meter register values.

        Original COM help: https://opendss.epri.com/Save.html
        """
        self._lib.Meters_Save()

    def SaveAll(self):
        """
        Save All EnergyMeter objects

        Original COM help: https://opendss.epri.com/SaveAll.html
        """
        self._lib.Meters_SaveAll()

    def SetActiveSection(self, SectIdx):
        self._lib.Meters_SetActiveSection(SectIdx)

    def AllBranchesInZone(self):
        """
        List (strings) of all branches in zone of the active EnergyMeter object.

        Original COM help: https://opendss.epri.com/AllBranchesInZone.html
        """
        return self._lib.Meters_Get_AllBranchesInZone()

    def AllEndElements(self):
        """
        Array of names of all zone end elements.

        Original COM help: https://opendss.epri.com/AllEndElements.html
        """
        return self._lib.Meters_Get_AllEndElements()

    def AllocFactors(self, *args):
        """
        Array of doubles: set the phase allocation factors for the active meter.

        Original COM help: https://opendss.epri.com/AllocFactors.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Meters_Get_AllocFactors_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Meters_Set_AllocFactors(ValuePtr, ValueCount)

    def AvgRepairTime(self):
        """
        Average Repair time in this section of the meter zone

        Original COM help: https://opendss.epri.com/AvgRepairTime.html
        """
        return self._lib.Meters_Get_AvgRepairTime()

    def CalcCurrent(self, *args):
        """
        Set the magnitude of the real part of the Calculated Current (normally determined by solution) for the Meter to force some behavior on Load Allocation

        Original COM help: https://opendss.epri.com/CalcCurrent.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Meters_Get_CalcCurrent_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Meters_Set_CalcCurrent(ValuePtr, ValueCount)

    def CountBranches(self):
        """
        Number of branches in Active EnergyMeter zone. (Same as sequence list size)

        Original COM help: https://opendss.epri.com/CountBranches.html
        """
        return self._lib.Meters_Get_CountBranches()

    def CountEndElements(self):
        """
        Number of zone end elements in the active meter zone.

        Original COM help: https://opendss.epri.com/CountEndElements.html
        """
        return self._lib.Meters_Get_CountEndElements()

    def CustInterrupts(self):
        """
        Total customer interruptions for this Meter zone based on reliability calcs.

        Original COM help: https://opendss.epri.com/CustInterrupts.html
        """
        return self._lib.Meters_Get_CustInterrupts()

    def DIFilesAreOpen(self):
        """
        Global Flag in the DSS to indicate if Demand Interval (DI) files have been properly opened.

        Original COM help: https://opendss.epri.com/DIFilesAreOpen.html
        """
        return self._lib.Meters_Get_DIFilesAreOpen()

    def FaultRateXRepairHrs(self):
        """
        Sum of Fault Rate time Repair Hrs in this section of the meter zone

        Original COM help: https://opendss.epri.com/FaultRateXRepairHrs.html
        """
        return self._lib.Meters_Get_FaultRateXRepairHrs()

    def MeteredElement(self, *args):
        """
        Name of metered element

        Original COM help: https://opendss.epri.com/MeteredElement.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Meters_Get_MeteredElement()

        # Setter
        (Value,) = args
        self._lib.Meters_Set_MeteredElement(Value)

    def MeteredTerminal(self, *args):
        """
        Number of Metered Terminal

        Original COM help: https://opendss.epri.com/MeteredTerminal.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Meters_Get_MeteredTerminal()

        # Setter
        (Value,) = args
        self._lib.Meters_Set_MeteredTerminal(Value)

    def NumSectionBranches(self):
        """
        Number of branches (lines) in this section

        Original COM help: https://opendss.epri.com/NumSectionBranches.html
        """
        return self._lib.Meters_Get_NumSectionBranches()

    def NumSectionCustomers(self):
        """
        Number of Customers in the active section.

        Original COM help: https://opendss.epri.com/NumSectionCustomers.html
        """
        return self._lib.Meters_Get_NumSectionCustomers()

    def NumSections(self):
        """
        Number of feeder sections in this meter's zone

        Original COM help: https://opendss.epri.com/NumSections.html
        """
        return self._lib.Meters_Get_NumSections()

    def OCPDeviceType(self):
        """
        Type of OCP device. 1=Fuse; 2=Recloser; 3=Relay

        Original COM help: https://opendss.epri.com/OCPDeviceType.html
        """
        return OCPDevTypeEnum(self._lib.Meters_Get_OCPDeviceType())

    def PeakCurrent(self, *args):
        """
        Array of doubles to set values of Peak Current property

        Original COM help: https://opendss.epri.com/Peakcurrent.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Meters_Get_Peakcurrent_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Meters_Set_Peakcurrent(ValuePtr, ValueCount)

    def RegisterNames(self):
        """
        Array of strings containing the names of the registers.

        See also the enum `EnergyMeterRegisters` for the standard register names.
        Besides those listed in the enumeration, users may need to check `RegisterNames`
        in order to find a specific register index at runtime.

        Original COM help: https://opendss.epri.com/RegisterNames1.html
        """
        return self._lib.Meters_Get_RegisterNames()

    def RegisterValues(self):
        """
        Array of all the values contained in the Meter registers for the active Meter.

        Original COM help: https://opendss.epri.com/RegisterValues1.html
        """
        return self._lib.Meters_Get_RegisterValues_GR()

    def SAIDI(self):
        """
        SAIDI for this meter's zone. Execute DoReliabilityCalc first.

        Original COM help: https://opendss.epri.com/SAIDI.html
        """
        return self._lib.Meters_Get_SAIDI()

    def SAIFI(self):
        """
        Returns SAIFI for this meter's Zone. Execute Reliability Calc method first.

        Original COM help: https://opendss.epri.com/SAIFI.html
        """
        return self._lib.Meters_Get_SAIFI()

    def SAIFIkW(self):
        """
        SAIFI based on kW rather than number of customers. Get after reliability calcs.

        Original COM help: https://opendss.epri.com/SAIFIKW.html
        """
        return self._lib.Meters_Get_SAIFIKW()

    def SectSeqidx(self):
        """
        SequenceIndex of the branch at the head of this section

        Original COM help: https://opendss.epri.com/SectSeqIdx.html
        """
        return self._lib.Meters_Get_SectSeqIdx()

    def SectTotalCust(self):
        """
        Total Customers downline from this section

        Original COM help: https://opendss.epri.com/SectTotalCust.html
        """
        return self._lib.Meters_Get_SectTotalCust()

    def SeqListSize(self):
        """
        Size of the Sequence List

        Original COM help: https://opendss.epri.com/SeqListSize.html
        """
        return self._lib.Meters_Get_SeqListSize()

    def SequenceList(self, *args):
        """
        Get/set Index into Meter's SequenceList that contains branch pointers in lexical order.
        Earlier index guaranteed to be upline from later index. Sets PDelement active.

        Original COM help: https://opendss.epri.com/SequenceIndex.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Meters_Get_SequenceIndex()

        # Setter
        (Value,) = args
        self._lib.Meters_Set_SequenceIndex(Value)

    def SumBranchFltRates(self):
        """
        Sum of the branch fault rates in this section of the meter's zone

        Original COM help: https://opendss.epri.com/SumBranchFltRates.html
        """
        return self._lib.Meters_Get_SumBranchFltRates()

    def TotalCustomers(self):
        """
        Total Number of customers in this zone (downline from the EnergyMeter)

        Original COM help: https://opendss.epri.com/TotalCustomers.html
        """
        return self._lib.Meters_Get_TotalCustomers()

    def Totals(self):
        """
        Totals of all registers of all meters

        Original COM help: https://opendss.epri.com/Totals.html
        """
        return self._lib.Meters_Get_Totals_GR()

    def ZonePCE(self):
        """
        Returns the list of all PCE within the area covered by the energy meter

        Original COM help: https://opendss.epri.com/ZonePCE.html
        """
        return self._lib.Meters_Get_ZonePCE()


_Meters = IMeters(api_util)

# For backwards compatibility, bind to the default instance
AllBranchesInZone = _Meters.AllBranchesInZone
AllEndElements = _Meters.AllEndElements
AllNames = _Meters.AllNames
AllocFactors = _Meters.AllocFactors
AvgRepairTime = _Meters.AvgRepairTime
CalcCurrent = _Meters.CalcCurrent
CloseAllDIFiles = _Meters.CloseAllDIFiles
Count = _Meters.Count
CountBranches = _Meters.CountBranches
CountEndElements = _Meters.CountEndElements
CustInterrupts = _Meters.CustInterrupts
DIFilesAreOpen = _Meters.DIFilesAreOpen
DoReliabilityCalc = _Meters.DoReliabilityCalc
FaultRateXRepairHrs = _Meters.FaultRateXRepairHrs
First = _Meters.First
Idx = _Meters.Idx
MeteredElement = _Meters.MeteredElement
MeteredTerminal = _Meters.MeteredTerminal
Name = _Meters.Name
Next = _Meters.Next
NumSectionBranches = _Meters.NumSectionBranches
NumSectionCustomers = _Meters.NumSectionCustomers
NumSections = _Meters.NumSections
OCPDeviceType = _Meters.OCPDeviceType
OpenAllDIFiles = _Meters.OpenAllDIFiles
PeakCurrent = _Meters.PeakCurrent
RegisterNames = _Meters.RegisterNames
RegisterValues = _Meters.RegisterValues
Reset = _Meters.Reset
ResetAll = _Meters.ResetAll
SAIDI = _Meters.SAIDI
SAIFI = _Meters.SAIFI
SAIFIkW = _Meters.SAIFIkW
Sample = _Meters.Sample
SampleAll = _Meters.SampleAll
Save = _Meters.Save
SaveAll = _Meters.SaveAll
SectSeqidx = _Meters.SectSeqidx
SectTotalCust = _Meters.SectTotalCust
SeqListSize = _Meters.SeqListSize
SequenceList = _Meters.SequenceList
SetActiveSection = _Meters.SetActiveSection
SumBranchFltRates = _Meters.SumBranchFltRates
TotalCustomers = _Meters.TotalCustomers
Totals = _Meters.Totals
ZonePCE = _Meters.ZonePCE
_columns = _Meters._columns
__all__ = [
    "AllBranchesInZone",
    "AllEndElements",
    "AllNames",
    "AllocFactors",
    "AvgRepairTime",
    "CalcCurrent",
    "CloseAllDIFiles",
    "Count",
    "CountBranches",
    "CountEndElements",
    "CustInterrupts",
    "DIFilesAreOpen",
    "DoReliabilityCalc",
    "FaultRateXRepairHrs",
    "First",
    "Idx",
    "MeteredElement",
    "MeteredTerminal",
    "Name",
    "Next",
    "NumSectionBranches",
    "NumSectionCustomers",
    "NumSections",
    "OCPDeviceType",
    "OpenAllDIFiles",
    "PeakCurrent",
    "RegisterNames",
    "RegisterValues",
    "Reset",
    "ResetAll",
    "SAIDI",
    "SAIFI",
    "SAIFIkW",
    "Sample",
    "SampleAll",
    "Save",
    "SaveAll",
    "SectSeqidx",
    "SectTotalCust",
    "SeqListSize",
    "SequenceList",
    "SetActiveSection",
    "SumBranchFltRates",
    "TotalCustomers",
    "Totals",
    "ZonePCE",
]



from typing import Any
from dss._cffi_api_util import Base as DSSPyBase

class Base(DSSPyBase):
    _oddpy = True
    
    def __init__(self, api_util):
        DSSPyBase.__init__(self, api_util)
        # Keep this since there are some differences when the array is empty.
        # self._get_string_array = api_util.get_string_array2
        self._frozen_attrs = True

    def __setattr__(self, __name: str, __value: Any) -> None:
        if self._frozen_attrs:
            raise AttributeError("Misuse of OpenDSSDirect.py: use function calls instead of assignments to interact with the DSS engine. For properties and COM-API compatibility, use DSS-Python.\nSee also https://dss-extensions.org/python_apis.html")
        
        object.__setattr__(self, __name, __value)


class Iterable(Base):
    """
    Provides iteration methods

    Based on DSS-Python's Iterable class.
    """ 

    __slots__ = [
        "_Get_First",
        "_Get_Next",
        "_Get_Count",
        "_Get_AllNames",
        "_Get_Name",
        "_Set_Name",
        "_Get_idx",
        "_Set_idx",
    ]
    
    def __init__(self, api_util):
        Base.__init__(self, api_util)
        
        prefix = self._api_prefix
        object.__setattr__(self, '_frozen_attrs', False)
        self._Get_First = getattr(self._lib, "{}_Get_First".format(prefix))
        self._Get_Next = getattr(self._lib, "{}_Get_Next".format(prefix))
        self._Get_Count = getattr(self._lib, "{}_Get_Count".format(prefix))
        self._Get_AllNames = getattr(self._lib, "{}_Get_AllNames".format(prefix))
        self._Get_Name = getattr(self._lib, "{}_Get_Name".format(prefix))
        self._Set_Name = getattr(self._lib, "{}_Set_Name".format(prefix))
        self._Get_idx = getattr(self._lib, "{}_Get_idx".format(prefix))
        self._Set_idx = getattr(self._lib, "{}_Set_idx".format(prefix))
        object.__setattr__(self, '_frozen_attrs', True)

    def First(self):
        """Sets the first object of this type active. Returns 0 if none."""
        return self._Get_First()

    def Next(self):
        """Sets next object of this type active. Returns 0 if no more."""
        return self._Get_Next()

    def Count(self):
        """Number of objects of this type"""
        return self._Get_Count()

    def __len__(self):
        """
        Number of objects of this type
        
        **(API Extension)**
        """
        return self._Get_Count()

    def __iter__(self):
        """
        Get an iterator of the object collection.
        
        Note that OpenDSS, via the classic APIs, only allow a single object of a specific type
        to be activated. That is, you cannot use references of distinct objects and interact
        with both at the same time, or keep a reference to use later. You need to reactivate
        the target object or ensure it is the active one.

        For an alternative, consider using our AltDSS-Python package.

        **(API Extension)**
        """
        idx = self._Get_First()
        while idx != 0:
            yield self
            idx = self._Get_Next()

    def AllNames(self):
        """Array of all names of this object type"""
        return self._Get_AllNames()

    def Name(self, *args):
        """Gets the current name or sets the active object of this type by name"""
        # Getter
        if len(args) == 0:
            return self._Get_Name()
        
        # Setter
        Value, = args        
        self._Set_Name(Value)
        
    def Idx(self, *args):
        """Gets the current index or sets the active object of this type by index"""
        # Getter
        if len(args) == 0:
            return self._Get_idx()

        # Setter
        Value, = args        
        if not isinstance(Value, (str, bytes)): #TODO: is this still required? Let it error out?
            self._Set_idx(Value)

__all__ = ["Iterable", "Base",]


from ._utils import api_util
from .Bases import Iterable
from dss import LineUnits


class ILines(Iterable):
    __slots__ = []

    __name__ = "Lines"
    _api_prefix = "Lines"
    _columns = [
        "Name",
        "Idx",
        "Phases",
        "Bus1",
        "Bus2",
        "LineCode",
        "Geometry",
        "Length",
        "IsSwitch",
        "Spacing",
        "EmergAmps",
        "NormAmps",
        "SeasonRating",
        "Yprim",
        "NumCust",
        "TotalCust",
        "Rho",
        "R0",
        "R1",
        "X0",
        "X1",
        "Rg",
        "Xg",
        "C0",
        "C1",
        "RMatrix",
        "XMatrix",
        "CMatrix",
        "Units",
    ]

    def New(self, Name):
        """Create new Line object with the given `Name`"""
        return self._lib.Lines_New(Name)

    def Bus1(self, *args):
        """
        Name of bus for terminal 1.

        Original COM help: https://opendss.epri.com/Bus1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Bus1()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Bus1(Value)

    def Bus2(self, *args):
        """
        Name of bus for terminal 2.

        Original COM help: https://opendss.epri.com/Bus2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Bus2()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Bus2(Value)

    def C0(self, *args):
        """
        Zero Sequence capacitance, nanofarads per unit length.

        Original COM help: https://opendss.epri.com/C0.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_C0()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_C0(Value)

    def C1(self, *args):
        """
        Positive Sequence capacitance, nanofarads per unit length.

        Original COM help: https://opendss.epri.com/C1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_C1()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_C1(Value)

    def CMatrix(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Cmatrix_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Lines_Set_Cmatrix(ValuePtr, ValueCount)

    def EmergAmps(self, *args):
        """
        Emergency (maximum) ampere rating of Line.

        Original COM help: https://opendss.epri.com/EmergAmps1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_EmergAmps()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_EmergAmps(Value)

    def Geometry(self, *args):
        """
        Line geometry code

        Original COM help: https://opendss.epri.com/Geometry.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Geometry()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Geometry(Value)

    def Length(self, *args):
        """
        Length of line in units compatible with the LineCode definition.

        Original COM help: https://opendss.epri.com/Length.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Length()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Length(Value)

    def LineCode(self, *args):
        """
        Name of LineCode object that defines the impedances.

        Original COM help: https://opendss.epri.com/LineCode.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_LineCode()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_LineCode(Value)

    def NormAmps(self, *args):
        """
        Normal ampere rating of Line.

        Original COM help: https://opendss.epri.com/NormAmps.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_NormAmps()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_NormAmps(Value)

    def NumCust(self):
        """
        Number of customers on this line.

        *Requires an energy meter with an updated zone.*

        Original COM help: https://opendss.epri.com/NumCust.html
        """
        return self._lib.Lines_Get_NumCust()

    def Parent(self):
        """
        Sets Parent of the active Line to be the active line. Returns 0 if no parent or action fails.

        *Requires an energy meter with an updated zone.*

        Original COM help: https://opendss.epri.com/Parent.html
        """
        return self._lib.Lines_Get_Parent()

    def Phases(self, *args):
        """
        Number of Phases, this Line element.

        Original COM help: https://opendss.epri.com/Phases1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Phases(Value)

    def R0(self, *args):
        """
        Zero Sequence resistance, ohms per unit length.

        Original COM help: https://opendss.epri.com/R0.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_R0()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_R0(Value)

    def R1(self, *args):
        """
        Positive Sequence resistance, ohms per unit length.

        Original COM help: https://opendss.epri.com/R1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_R1()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_R1(Value)

    def Rg(self, *args):
        """
        Earth return resistance value used to compute line impedances at power frequency

        Original COM help: https://opendss.epri.com/Rg.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Rg()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Rg(Value)

    def Rho(self, *args):
        """
        Earth Resistivity, m-ohms

        Original COM help: https://opendss.epri.com/Rho.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Rho()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Rho(Value)

    def RMatrix(self, *args):
        """
        Resistance matrix (full), ohms per unit length. Array of doubles.

        Original COM help: https://opendss.epri.com/Rmatrix.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Rmatrix_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Lines_Set_Rmatrix(ValuePtr, ValueCount)

    def Spacing(self, *args):
        """
        Line spacing code

        Original COM help: https://opendss.epri.com/Spacing.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Spacing()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Spacing(Value)

    def TotalCust(self):
        """
        Total Number of customers served from this line.

        Original COM help: https://opendss.epri.com/TotalCust.html
        """
        return self._lib.Lines_Get_TotalCust()

    def Units(self, *args):
        """
        Length units for the active line.

        Original COM help: https://opendss.epri.com/Units.html
        """
        # Getter
        if len(args) == 0:
            return LineUnits(self._lib.Lines_Get_Units())

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Units(Value)

    def X0(self, *args):
        """
        Zero Sequence reactance ohms per unit length.

        Original COM help: https://opendss.epri.com/X0.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_X0()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_X0(Value)

    def X1(self, *args):
        """
        Positive Sequence reactance, ohms per unit length.

        Original COM help: https://opendss.epri.com/X1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_X1()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_X1(Value)

    def Xg(self, *args):
        """
        Earth return reactance value used to compute line impedances at power frequency

        Original COM help: https://opendss.epri.com/Xg.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Xg()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_Xg(Value)

    def XMatrix(self, *args):
        """
        Reactance matrix (full), ohms per unit length. Array of doubles.

        Original COM help: https://opendss.epri.com/Xmatrix.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Xmatrix_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Lines_Set_Xmatrix(ValuePtr, ValueCount)

    def Yprim(self, *args):
        """
        Yprimitive for the active line object (complex array).

        Original COM help: https://opendss.epri.com/Yprim1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_Yprim_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Lines_Set_Yprim(ValuePtr, ValueCount)

    def SeasonRating(self):
        """
        Delivers the rating for the current season (in Amps)  if the "SeasonalRatings" option is active

        Original COM help: https://opendss.epri.com/SeasonRating.html
        """
        return self._lib.Lines_Get_SeasonRating()

    def IsSwitch(self, *args):
        """
        Line element switch status. Setting it has side-effects to the line parameters.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.Lines_Get_IsSwitch()

        # Setter
        (Value,) = args
        self._lib.Lines_Set_IsSwitch(Value)


_Lines = ILines(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Lines.AllNames
Bus1 = _Lines.Bus1
Bus2 = _Lines.Bus2
C0 = _Lines.C0
C1 = _Lines.C1
CMatrix = _Lines.CMatrix
Count = _Lines.Count
EmergAmps = _Lines.EmergAmps
First = _Lines.First
Geometry = _Lines.Geometry
Idx = _Lines.Idx
IsSwitch = _Lines.IsSwitch
Length = _Lines.Length
LineCode = _Lines.LineCode
Name = _Lines.Name
New = _Lines.New
Next = _Lines.Next
NormAmps = _Lines.NormAmps
NumCust = _Lines.NumCust
Parent = _Lines.Parent
Phases = _Lines.Phases
R0 = _Lines.R0
R1 = _Lines.R1
RMatrix = _Lines.RMatrix
Rg = _Lines.Rg
Rho = _Lines.Rho
SeasonRating = _Lines.SeasonRating
Spacing = _Lines.Spacing
TotalCust = _Lines.TotalCust
Units = _Lines.Units
X0 = _Lines.X0
X1 = _Lines.X1
XMatrix = _Lines.XMatrix
Xg = _Lines.Xg
Yprim = _Lines.Yprim
_columns = _Lines._columns
__all__ = [
    "AllNames",
    "Bus1",
    "Bus2",
    "C0",
    "C1",
    "CMatrix",
    "Count",
    "EmergAmps",
    "First",
    "Geometry",
    "Idx",
    "IsSwitch",
    "Length",
    "LineCode",
    "Name",
    "New",
    "Next",
    "NormAmps",
    "NumCust",
    "Parent",
    "Phases",
    "R0",
    "R1",
    "RMatrix",
    "Rg",
    "Rho",
    "SeasonRating",
    "Spacing",
    "TotalCust",
    "Units",
    "X0",
    "X1",
    "XMatrix",
    "Xg",
    "Yprim",
]



from ._utils import api_util
from .Bases import Iterable


class IIsource(Iterable):
    __slots__ = []

    __name__ = "Isource"
    _api_prefix = "ISources"
    _columns = ["Name", "Idx", "Amps", "AngleDeg", "Frequency"]

    def Amps(self, *args):
        """
        Magnitude of the ISource in amps

        Original COM help: https://opendss.epri.com/Amps.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ISources_Get_Amps()

        # Setter
        (Value,) = args
        self._lib.ISources_Set_Amps(Value)

    def AngleDeg(self, *args):
        """
        Phase angle for ISource, degrees

        Original COM help: https://opendss.epri.com/AngleDeg.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ISources_Get_AngleDeg()

        # Setter
        (Value,) = args
        self._lib.ISources_Set_AngleDeg(Value)

    def Frequency(self, *args):
        """
        The present frequency of the ISource, Hz

        Original COM help: https://opendss.epri.com/Frequency.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ISources_Get_Frequency()

        # Setter
        (Value,) = args
        self._lib.ISources_Set_Frequency(Value)


_Isource = IIsource(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Isource.AllNames
Amps = _Isource.Amps
AngleDeg = _Isource.AngleDeg
Count = _Isource.Count
First = _Isource.First
Frequency = _Isource.Frequency
Idx = _Isource.Idx
Name = _Isource.Name
Next = _Isource.Next
_columns = _Isource._columns
__all__ = [
    "AllNames",
    "Amps",
    "AngleDeg",
    "Count",
    "First",
    "Frequency",
    "Idx",
    "Name",
    "Next",
]



from ._utils import api_util
from .Bases import Iterable
from dss import LineUnits


class ILineSpacings(Iterable):
    """
    LineSpacing objects

    (API Extension)
    """

    __name__ = "LineSpacings"
    _api_prefix = "LineSpacings"
    __slots__ = []
    _columns = ["Name", "Idx", "Nconds", "Phases", "Units", "Xcoords", "Ycoords"]

    def Phases(self, *args):
        """Number of Phases"""
        # Getter
        if len(args) == 0:
            return self._lib.LineSpacings_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.LineSpacings_Set_Phases(Value)

    def Nconds(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.LineSpacings_Get_Nconds()

        # Setter
        (Value,) = args
        self._lib.LineSpacings_Set_Nconds(Value)

    def Units(self, *args):
        # Getter
        if len(args) == 0:
            return LineUnits(self._lib.LineSpacings_Get_Units())

        # Setter
        (Value,) = args
        self._lib.LineSpacings_Set_Units(Value)

    def Xcoords(self, *args):
        """Get/Set the X (horizontal) coordinates of the conductors"""
        # Getter
        if len(args) == 0:
            return self._lib.LineSpacings_Get_Xcoords_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineSpacings_Set_Xcoords(ValuePtr, ValueCount)

    def Ycoords(self, *args):
        """Get/Set the Y (vertical/height) coordinates of the conductors"""
        # Getter
        if len(args) == 0:
            return self._lib.LineSpacings_Get_Ycoords_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineSpacings_Set_Ycoords(ValuePtr, ValueCount)


_LineSpacings = ILineSpacings(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _LineSpacings.AllNames
Count = _LineSpacings.Count
First = _LineSpacings.First
Idx = _LineSpacings.Idx
Name = _LineSpacings.Name
Nconds = _LineSpacings.Nconds
Next = _LineSpacings.Next
Phases = _LineSpacings.Phases
Units = _LineSpacings.Units
Xcoords = _LineSpacings.Xcoords
Ycoords = _LineSpacings.Ycoords
_columns = _LineSpacings._columns
__all__ = [
    "AllNames",
    "Count",
    "First",
    "Idx",
    "Name",
    "Nconds",
    "Next",
    "Phases",
    "Units",
    "Xcoords",
    "Ycoords",
]



from ._utils import api_util
from .Bases import Base


class IProgress(Base):
    __slots__ = []

    __name__ = "Progress"
    _api_prefix = "DSSProgress"
    _columns = []

    def Close(self):
        """
        Close progress form

        Typically used with EPRI's OpenDSS, on Windows. Otherwise, it could be a no-op.
        """
        self._lib.DSSProgress_Close()

    def Show(self):
        """
        Show progress form

        Typically used with EPRI's OpenDSS, on Windows. Otherwise, it could be a no-op.
        """
        self._lib.DSSProgress_Show()

    def Caption(self, Value):
        """
        Set the caption to appear on the bottom of the DSS Progress form.

        Typically used with EPRI's OpenDSS, on Windows. Otherwise, it could be a no-op.

        Original COM help: https://opendss.epri.com/Caption.html
        """
        self._lib.DSSProgress_Set_Caption(Value)

    def PctProgress(self, Value):
        """
        Set the percent progress to indicate [0..100] on the progress form.

        Typically used with EPRI's OpenDSS, on Windows. Otherwise, it could be a no-op.

        Original COM help: https://opendss.epri.com/PctProgress.html
        """
        self._lib.DSSProgress_Set_PctProgress(Value)


_Progress = IProgress(api_util)

# For backwards compatibility, bind to the default instance
Caption = _Progress.Caption
Close = _Progress.Close
PctProgress = _Progress.PctProgress
Show = _Progress.Show
_columns = _Progress._columns
__all__ = ["Caption", "Close", "PctProgress", "Show"]



from ._utils import api_util
from .Bases import Iterable
from dss_python_backend.enums import LineUnits

class ICNData(Iterable):
    """
    CNData objects

    *Not available when using EPRI's OpenDSS distribution.*

    **(API Extension)**
    """

    __name__ = "CNData"
    _api_prefix = "CNData"
    __slots__ = []
    _columns = [
        "Name",
        "Idx",
        "NormAmps",
        "EmergAmps",
        "Rdc",
        "Rac",
        "ResistanceUnits",
        "GMRac",
        "GMRUnits",
        "Radius",
        "Diameter",
        "RadiusUnits",
        "EpsR",
        "InsLayer",
        "DiaIns",
        "DiaCable",
        "DiaStrand",
        "RStrand",
        "k",
    ]

    def EmergAmps(self, *args):
        """Emergency ampere rating"""
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_EmergAmps()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_EmergAmps(Value)

    def NormAmps(self, *args):
        """Normal Ampere rating"""
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_NormAmps()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_NormAmps(Value)

    def Rdc(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_Rdc()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_Rdc(Value)

    def Rac(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_Rac()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_Rac(Value)

    def GMRac(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_GMRac()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_GMRac(Value)

    def GMRUnits(self, *args):
        # Getter
        if len(args) == 0:
            return LineUnits(self._lib.CNData_Get_GMRUnits())

        # Setter
        (Value,) = args
        self._lib.CNData_Set_GMRUnits(Value)

    def Radius(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_Radius()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_Radius(Value)

    def RadiusUnits(self, *args):
        # Getter
        if len(args) == 0:
            return LineUnits(self._lib.CNData_Get_RadiusUnits())

        # Setter
        (Value,) = args
        self._lib.CNData_Set_RadiusUnits(Value)

    def ResistanceUnits(self, *args):
        # Getter
        if len(args) == 0:
            return LineUnits(self._lib.CNData_Get_ResistanceUnits())

        # Setter
        (Value,) = args
        self._lib.CNData_Set_ResistanceUnits(Value)

    def Diameter(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_Diameter()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_Diameter(Value)

    def EpsR(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_EpsR()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_EpsR(Value)

    def InsLayer(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_InsLayer()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_InsLayer(Value)

    def DiaIns(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_DiaIns()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_DiaIns(Value)

    def DiaCable(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_DiaCable()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_DiaCable(Value)

    def k(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_k()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_k(Value)

    def DiaStrand(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_DiaStrand()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_DiaStrand(Value)

    def GmrStrand(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_GmrStrand()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_GmrStrand(Value)

    def RStrand(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.CNData_Get_RStrand()

        # Setter
        (Value,) = args
        self._lib.CNData_Set_RStrand(Value)


_CNData = ICNData(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _CNData.AllNames
Count = _CNData.Count
DiaCable = _CNData.DiaCable
DiaIns = _CNData.DiaIns
DiaStrand = _CNData.DiaStrand
Diameter = _CNData.Diameter
EmergAmps = _CNData.EmergAmps
EpsR = _CNData.EpsR
First = _CNData.First
GMRUnits = _CNData.GMRUnits
GMRac = _CNData.GMRac
GmrStrand = _CNData.GmrStrand
Idx = _CNData.Idx
InsLayer = _CNData.InsLayer
Name = _CNData.Name
Next = _CNData.Next
NormAmps = _CNData.NormAmps
RStrand = _CNData.RStrand
Rac = _CNData.Rac
Radius = _CNData.Radius
RadiusUnits = _CNData.RadiusUnits
Rdc = _CNData.Rdc
ResistanceUnits = _CNData.ResistanceUnits
k = _CNData.k
_columns = _CNData._columns
__all__ = [
    "AllNames",
    "Count",
    "DiaCable",
    "DiaIns",
    "DiaStrand",
    "Diameter",
    "EmergAmps",
    "EpsR",
    "First",
    "GMRUnits",
    "GMRac",
    "GmrStrand",
    "Idx",
    "InsLayer",
    "Name",
    "Next",
    "NormAmps",
    "RStrand",
    "Rac",
    "Radius",
    "RadiusUnits",
    "Rdc",
    "ResistanceUnits",
    "k",
]
